@@ -23,7 +23,10 @@
 import sys, os, time
 import GNS3.Globals as globals
 import GNS3.Config.Defaults as Defaults
-from PyQt4.QtGui import QApplication, QSplashScreen, QPixmap
+import GNS3.Dynagen.dynamips_lib as lib
+import GNS3.Dynagen.portTracker_lib as tracker
+from distutils.version import LooseVersion
+from PyQt4.QtGui import QApplication, QSplashScreen, QPixmap, QMessageBox
 from PyQt4.QtCore import Qt, QVariant, QSettings, QEventLoop
 from GNS3.Utils import Singleton, translate
 from GNS3.Workspace import Workspace
@@ -37,7 +40,7 @@ from GNS3.Translations import Translator
 from GNS3.DynagenSub import DynagenSub
 from GNS3.ProjectDialog import ProjectDialog
 from GNS3.Wizard import Wizard
-from __main__ import VERSION_INTEGER
+from __main__ import VERSION
 
 class Application(QApplication, Singleton):
     """ GNS3 Application instance
@@ -407,24 +410,22 @@ class Application(QApplication, Singleton):
         else:
             confo.memory_limit = int(ConfDB().get("Dynamips/hypervisor_memory_usage_limit", 1024))
         confo.udp_incrementation = int(ConfDB().get("Dynamips/hypervisor_udp_incrementation", 100))
+        confo.detected_version = ConfDB().get('Dynamips/detected_version', unicode(''))
         confo.import_use_HypervisorManager = ConfDB().value("Dynamips/hypervisor_manager_import", QVariant(True)).toBool()
         confo.HypervisorManager_binding = ConfDB().get('Dynamips/hypervisor_manager_binding', unicode('127.0.0.1'))
         confo.allocateHypervisorPerIOS = ConfDB().value("Dynamips/allocate_hypervisor_per_IOS", QVariant(True)).toBool()
 
-        # replace ~user and $HOME by home directory
-        if os.environ.has_key("HOME"):
-            confo.path = confo.path.replace('$HOME', os.environ["HOME"])
-            confo.workdir =  confo.workdir.replace('$HOME', os.environ["HOME"])
-        confo.path = os.path.expanduser(confo.path)
-        confo.workdir = os.path.expanduser(confo.workdir)
+        # Expand user home dir and environment variables
+        confo.path = os.path.expandvars(os.path.expanduser(confo.path))
+        confo.workdir = os.path.expandvars(os.path.expanduser(confo.workdir))
 
         # Qemu config
         self.systconf['qemu'] = systemQemuConf()
         confo = self.systconf['qemu']
         confo.qemuwrapper_path = ConfDB().get('Qemu/qemuwrapper_path', Defaults.QEMUWRAPPER_DEFAULT_PATH)
         confo.qemuwrapper_workdir = ConfDB().get('Qemu/qemuwrapper_working_directory', Defaults.QEMUWRAPPER_DEFAULT_WORKDIR)
-        confo.qemu_path = ConfDB().get('Qemu/qemu_path', unicode('/usr/local/bin/qemu-system-i386'))
-        confo.qemu_img_path = ConfDB().get('Qemu/qemu_img_path', unicode('/usr/local/bin/qemu-img'))
+        confo.qemu_path = ConfDB().get('Qemu/qemu_path', Defaults.QEMU_DEFAULT_PATH)
+        confo.qemu_img_path = ConfDB().get('Qemu/qemu_img_path', Defaults.QEMU_IMG_DEFAULT_PATH)
         confo.external_hosts = ConfDB().get('Qemu/external_hosts', unicode('localhost:10525')).split(',')
         confo.enable_QemuWrapperAdvOptions = ConfDB().value("Qemu/enable_QemuWrapperAdvOptions", QVariant(False)).toBool()
         confo.enable_QemuManager = ConfDB().value("Qemu/enable_QemuManager", QVariant(True)).toBool()
@@ -432,24 +433,20 @@ class Application(QApplication, Singleton):
         confo.send_path_external_QemuWrapper = ConfDB().value("Qemu/send_paths_external_Qemuwrapper", QVariant(False)).toBool()
         confo.QemuManager_binding = ConfDB().get('Qemu/qemu_manager_binding', unicode('127.0.0.1'))
         confo.qemuwrapper_port = int(ConfDB().get('Qemu/qemuwrapper_port', 10525))
-        confo.qemuwrapper_baseUDP = int(ConfDB().get('Qemu/qemuwrapper_baseUDP', 20000))
-        confo.qemuwrapper_baseConsole = int(ConfDB().get('Qemu/qemuwrapper_baseConsole', 3000))
+        confo.qemuwrapper_baseUDP = int(ConfDB().get('Qemu/qemuwrapper_baseUDP', 40000))
+        confo.qemuwrapper_baseConsole = int(ConfDB().get('Qemu/qemuwrapper_baseConsole', 3001))
 
-        # Qemu replace ~user and $HOME by home directory
-        if os.environ.has_key("HOME"):
-            confo.qemuwrapper_path = confo.qemuwrapper_path.replace('$HOME', os.environ["HOME"])
-            confo.qemuwrapper_workdir =  confo.qemuwrapper_workdir.replace('$HOME', os.environ["HOME"])
-
-        confo.qemuwrapper_path = os.path.expanduser(confo.qemuwrapper_path)
-        confo.qemuwrapper_workdir = os.path.expanduser(confo.qemuwrapper_workdir)
+        # Expand user home dir and environment variables
+        confo.qemuwrapper_path = os.path.expandvars(os.path.expanduser(confo.qemuwrapper_path))
+        confo.qemuwrapper_workdir = os.path.expandvars(os.path.expanduser(confo.qemuwrapper_workdir))
 
         # VBox config
-
         self.systconf['vbox'] = systemVBoxConf()
         confo = self.systconf['vbox']
         confo.vboxwrapper_path = ConfDB().get('VBox/vboxwrapper_path', Defaults.VBOXWRAPPER_DEFAULT_PATH)
         confo.vboxwrapper_workdir = ConfDB().get('VBox/vboxwrapper_working_directory', Defaults.VBOXWRAPPER_DEFAULT_WORKDIR)
         confo.external_hosts = ConfDB().get('VBox/external_hosts', unicode('localhost:11525')).split(',')
+        confo.use_VBoxVmnames = ConfDB().value("VBox/use_VBoxVmnames", QVariant(True)).toBool()
         confo.enable_VBoxWrapperAdvOptions = ConfDB().value("VBox/enable_VBoxWrapperAdvOptions", QVariant(False)).toBool()
         confo.enable_VBoxAdvOptions = ConfDB().value("VBox/enable_VBoxAdvOptions", QVariant(False)).toBool()
         confo.enable_GuestControl = ConfDB().value("VBox/enable_GuestControl", QVariant(False)).toBool()
@@ -458,15 +455,11 @@ class Application(QApplication, Singleton):
         confo.VBoxManager_binding = ConfDB().get('VBox/vbox_manager_binding', unicode('127.0.0.1'))
         confo.vboxwrapper_port = int(ConfDB().get('VBox/vboxwrapper_port', 11525))
         confo.vboxwrapper_baseUDP = int(ConfDB().get('VBox/vboxwrapper_baseUDP', 20900))
-        confo.vboxwrapper_baseConsole = int(ConfDB().get('VBox/vboxwrapper_baseConsole', 3900))
+        confo.vboxwrapper_baseConsole = int(ConfDB().get('VBox/vboxwrapper_baseConsole', 3501))
 
-        # VBox replace ~user and $HOME by home directory
-        if os.environ.has_key("HOME"):
-            confo.vboxwrapper_path = confo.vboxwrapper_path.replace('$HOME', os.environ["HOME"])
-            confo.vboxwrapper_workdir =  confo.vboxwrapper_workdir.replace('$HOME', os.environ["HOME"])
-
-        confo.vboxwrapper_path = os.path.expanduser(confo.vboxwrapper_path)
-        confo.vboxwrapper_workdir = os.path.expanduser(confo.vboxwrapper_workdir)
+        # Expand user home dir and environment variables
+        confo.vboxwrapper_path = os.path.expandvars(os.path.expanduser(confo.vboxwrapper_path))
+        confo.vboxwrapper_workdir = os.path.expandvars(os.path.expanduser(confo.vboxwrapper_workdir))
 
         # Capture config
         self.systconf['capture'] = systemCaptureConf()
@@ -475,24 +468,26 @@ class Application(QApplication, Singleton):
         confo.cap_cmd = ConfDB().get('Capture/capture_reader_cmd', Defaults.CAPTURE_DEFAULT_CMD)
         confo.auto_start = ConfDB().value('Capture/auto_start_cmd', QVariant(False)).toBool()
 
-        # replace ~user and $HOME by home directory
-        if os.environ.has_key("HOME"):
-            confo.cap_cmd = confo.cap_cmd.replace('$HOME', os.environ["HOME"])
-            confo.workdir =  confo.workdir.replace('$HOME', os.environ["HOME"])
-        confo.cap_cmd = os.path.expanduser(confo.cap_cmd)
-        confo.workdir = os.path.expanduser(confo.workdir)
+        # Expand user home dir and environment variables
+        confo.cap_cmd = os.path.expandvars(os.path.expanduser(confo.cap_cmd))
+        confo.workdir = os.path.expandvars(os.path.expanduser(confo.workdir))
 
         # System general config
         self.systconf['general'] = systemGeneralConf()
         confo = self.systconf['general']
         confo.lang = ConfDB().get('GNS3/lang', unicode('en'))
-        confo.slow_start = int(ConfDB().get('GNS3/slow_start', 0))
+        confo.slow_start = int(ConfDB().get('GNS3/slow_start', 1))
         confo.autosave = int(ConfDB().get('GNS3/autosave', 0))
         confo.project_startup = ConfDB().value("GNS3/project_startup", QVariant(True)).toBool()
         confo.relative_paths = ConfDB().value("GNS3/relative_paths", QVariant(True)).toBool()
-        confo.use_shell = ConfDB().value("GNS3/use_shell", QVariant(True)).toBool()
+        if sys.platform.startswith('win'):
+            confo.use_shell = ConfDB().value("GNS3/use_shell", QVariant(False)).toBool()
+        else:
+            confo.use_shell = ConfDB().value("GNS3/use_shell", QVariant(True)).toBool()
         confo.bring_console_to_front = ConfDB().value("GNS3/bring_console_to_front", QVariant(False)).toBool()
         confo.term_cmd = ConfDB().get('GNS3/console', Defaults.TERMINAL_DEFAULT_CMD)
+        confo.term_serial_cmd = ConfDB().get('GNS3/serial_console', Defaults.TERMINAL_SERIAL_DEFAULT_CMD)
+        confo.term_close_on_delete = ConfDB().value("GNS3/term_close_on_delete", QVariant(False)).toBool()
         confo.project_path = ConfDB().get('GNS3/project_directory', Defaults.PROJECT_DEFAULT_DIR)
         confo.ios_path = ConfDB().get('GNS3/ios_directory', Defaults.IOS_DEFAULT_DIR)
         confo.status_points = ConfDB().value("GNS3/gui_show_status_points", QVariant(True)).toBool()
@@ -508,14 +503,16 @@ class Application(QApplication, Singleton):
             confo.auto_check_for_update = ConfDB().value("GNS3/auto_check_for_update", QVariant(False)).toBool()
         confo.last_check_for_update = int(ConfDB().get('GNS3/last_check_for_update', 0))
 
-        # replace ~user and $HOME by home directory
-        if os.environ.has_key("HOME"):
-            confo.term_cmd = confo.term_cmd.replace('$HOME', os.environ["HOME"])
-            confo.project_path = confo.project_path.replace('$HOME', os.environ["HOME"])
-            confo.ios_path =  confo.ios_path.replace('$HOME', os.environ["HOME"])
-        confo.term_cmd = os.path.expanduser(confo.term_cmd)
-        confo.project_path = os.path.expanduser(confo.project_path)
-        confo.ios_path = os.path.expanduser(confo.ios_path)
+        # Expand user home dir and environment variables
+        confo.term_cmd = os.path.expandvars(os.path.expanduser(confo.term_cmd))
+        confo.project_path = os.path.expandvars(os.path.expanduser(confo.project_path))
+        confo.ios_path = os.path.expandvars(os.path.expanduser(confo.ios_path))
+
+        # Restore debug level
+        globals.debugLevel = int(ConfDB().get('GNS3/debug_level', 0))
+        if globals.debugLevel == 1 or globals.debugLevel == 3:
+            lib.setdebug(True)
+            tracker.setdebug(True)
 
         # Now systGeneral settings are loaded, load the translator
         self.translator = Translator()
@@ -561,21 +558,34 @@ class Application(QApplication, Singleton):
         self.mainWindow.restoreState(ConfDB().value("GUIState/State").toByteArray())
         self.mainWindow.show()
 
-        configFile = unicode(ConfDB().fileName(), 'utf-8', errors='replace')
-        # if we cannot find our config file, we start the Wizard dialog
-        if not os.access(configFile, os.F_OK):
-            dialog = Wizard(parent=self.mainWindow)
-            dialog.show()
-            self.mainWindow.centerDialog(dialog)
-            dialog.raise_()
-            dialog.activateWindow()
+        force_clear_configuration = True
+        version = ConfDB().get('GNS3/version', '0.0.1')
+        try:
+            # trick to test old version format (integer), before 0.8.2.1 release
+            int(version)
+        except:
+            force_clear_configuration = False
+            pass
+            # for future releases
+            #if LooseVersion(VERSION) > version:
+            #    pass
+
+        if force_clear_configuration:
             self.mainWindow.raise_()
-            dialog.raise_()
+            reply = QMessageBox.question(self.mainWindow, translate("Application", "GNS3 configuration file"),
+                                        translate("Application", "You have installed a new GNS3 version.\nIt is recommended to clear your old configuration, do you want to proceed?"), QMessageBox.Yes, QMessageBox.No)
+            if reply == QMessageBox.Yes:
+                ConfDB().clear()
+                c = ConfDB()
+                c.set('GNS3/version', VERSION)
+                c.sync()
+                globals.recordConfiguration = False
+                QMessageBox.information(self.mainWindow, translate("Application", "GNS3 configuration file"), translate("Application", "Configuration cleared!\nPlease restart GNS3"))
         else:
-            if file:
-                self.mainWindow.load_netfile(file)
-            elif confo.project_startup and os.access(configFile, os.F_OK):
-                dialog = ProjectDialog(parent=self.mainWindow, newProject=True)
+            # if we cannot find our config file, we start the Wizard dialog
+            configFile = unicode(ConfDB().fileName(), 'utf-8', errors='replace')
+            if not os.access(configFile, os.F_OK):
+                dialog = Wizard(parent=self.mainWindow)
                 dialog.show()
                 self.mainWindow.centerDialog(dialog)
                 dialog.raise_()
@@ -583,7 +593,18 @@ class Application(QApplication, Singleton):
                 self.mainWindow.raise_()
                 dialog.raise_()
             else:
-                self.mainWindow.raise_()
+                if file:
+                    self.mainWindow.load_netfile(file)
+                elif confo.project_startup and os.access(configFile, os.F_OK):
+                    dialog = ProjectDialog(parent=self.mainWindow, newProject=True)
+                    dialog.show()
+                    self.mainWindow.centerDialog(dialog)
+                    dialog.raise_()
+                    dialog.activateWindow()
+                    self.mainWindow.raise_()
+                    dialog.raise_()
+                else:
+                    self.mainWindow.raise_()
 
         retcode = QApplication.exec_()
 
@@ -604,7 +625,7 @@ class Application(QApplication, Singleton):
         """
 
         c = ConfDB()
-        c.set('GNS3/version', VERSION_INTEGER)
+        c.set('GNS3/version', VERSION)
 
         # Apply general settings
         confo = self.systconf['general']
@@ -614,6 +635,8 @@ class Application(QApplication, Singleton):
         c.set('GNS3/slow_start', confo.slow_start)
         c.set('GNS3/autosave', confo.autosave)
         c.set('GNS3/console', confo.term_cmd)
+        c.set('GNS3/serial_console', confo.term_serial_cmd)
+        c.set('GNS3/term_close_on_delete', confo.term_close_on_delete)
         c.set('GNS3/use_shell', confo.use_shell)
         c.set('GNS3/bring_console_to_front', confo.bring_console_to_front)
         c.set('GNS3/gui_show_status_points', confo.status_points)
@@ -626,6 +649,7 @@ class Application(QApplication, Singleton):
         c.set('GNS3/auto_check_for_update', confo.auto_check_for_update)
         c.set('GNS3/last_check_for_update', confo.last_check_for_update)
         c.set('GNS3/console_delay', confo.console_delay)
+        c.set('GNS3/debug_level', globals.debugLevel)
 
         # Dynamips settings
         confo = self.systconf['dynamips']
@@ -641,6 +665,7 @@ class Application(QApplication, Singleton):
         c.set('Dynamips/dynamips_jitsharing', confo.jitsharing)
         c.set('Dynamips/dynamips_mmap', confo.mmap)
         c.set('Dynamips/hypervisor_memory_usage_limit', confo.memory_limit)
+        c.set('Dynamips/detected_version', confo.detected_version)
         c.set('Dynamips/hypervisor_udp_incrementation', confo.udp_incrementation)
         c.set('Dynamips/hypervisor_manager_import', confo.import_use_HypervisorManager)
         c.set('Dynamips/allocate_hypervisor_per_IOS', confo.allocateHypervisorPerIOS)
@@ -669,6 +694,7 @@ class Application(QApplication, Singleton):
         c.set('VBox/vboxwrapper_working_directory', confo.vboxwrapper_workdir)
         external_hosts = ','.join(confo.external_hosts)
         c.set('VBox/external_hosts', external_hosts)
+        c.set('VBox/use_VBoxVmnames', confo.use_VBoxVmnames)
         c.set('VBox/enable_VBoxWrapperAdvOptions', confo.enable_VBoxWrapperAdvOptions)
         c.set('VBox/enable_VBoxAdvOptions', confo.enable_VBoxAdvOptions)
         c.set('VBox/enable_GuestControl', confo.enable_GuestControl)
@@ -776,6 +802,10 @@ class Application(QApplication, Singleton):
             c.set(basekey + "/filename", o.filename)
             c.set(basekey + "/nic_nb", o.nic_nb)
             c.set(basekey + "/nic", o.nic)
+            c.set(basekey + "/first_nic_managed", o.first_nic_managed)
+            c.set(basekey + "/headless_mode", o.headless_mode)
+            c.set(basekey + "/console_support", o.console_support)
+            c.set(basekey + "/console_telnet_server", o.console_telnet_server)
             c.set(basekey + "/guestcontrol_user", o.guestcontrol_user)
             c.set(basekey + "/guestcontrol_password", o.guestcontrol_password)
 
