@@ -21,7 +21,6 @@
 
 import os, sys, platform
 from GNS3.Utils import translate
-from __main__ import GNS3_RUN_PATH
 
 # Default path to Dynamips executable
 if sys.platform.startswith('win'):
@@ -56,8 +55,10 @@ else:
     qemuwrapper_path = os.getcwdu() + os.sep + 'qemuwrapper/qemuwrapper.py'
     if os.path.exists(qemuwrapper_path):
         QEMUWRAPPER_DEFAULT_PATH = qemuwrapper_path
+    elif platform.system() == 'Linux':
+        QEMUWRAPPER_DEFAULT_PATH = unicode("/usr/lib/gns3/qemuwrapper.py")
     else:
-        QEMUWRAPPER_DEFAULT_PATH = unicode("/usr/local/libexec/gns3/qemuwrapper.py")
+        QEMUWRAPPER_DEFAULT_PATH = unicode("/usr/local/libexec/gns3/qemuwrapper.py") # libexec is standard on BSD platforms
 
 # Default path to qemuwrapper working directory
 if os.environ.has_key("TEMP"):
@@ -67,11 +68,19 @@ elif os.environ.has_key("TMP"):
 else:
     QEMUWRAPPER_DEFAULT_WORKDIR = unicode('/tmp')
 
+# Default paths to Qemu and qemu-img
+if sys.platform.startswith('win'):
+    QEMU_DEFAULT_PATH = unicode('qemu')
+    QEMU_IMG_DEFAULT_PATH = unicode('qemu-img')
+else:
+    QEMU_DEFAULT_PATH = unicode('/usr/local/bin/qemu-system-i386')
+    QEMU_IMG_DEFAULT_PATH = unicode('/usr/local/bin/qemu-img')
+
 # Default path to vboxwrapper
 if sys.platform.startswith('win'):
     VBOXWRAPPER_DEFAULT_PATH = unicode('vboxwrapper.exe')
 elif sys.platform.startswith('darwin') and hasattr(sys, "frozen"):
-    VBOXWRAPPER_DEFAULT_PATH = os.getcwdu() + os.sep + '../Resources/vboxwrapper.py'
+    VBOXWRAPPER_DEFAULT_PATH = os.getcwdu() + os.sep + '../Resources/VBoxWrapper/Contents/MacOS/VBoxWrapper'
 elif sys.platform.startswith('openbsd'):
     VBOXWRAPPER_DEFAULT_PATH = unicode('/usr/local/libexec/gns3/vboxwrapper.py')
 else:
@@ -79,8 +88,10 @@ else:
     vboxwrapper_path = os.getcwdu() + os.sep + 'vboxwrapper/vboxwrapper.py'
     if os.path.exists(vboxwrapper_path):
         VBOXWRAPPER_DEFAULT_PATH = vboxwrapper_path
+    elif platform.system() == 'Linux':
+        VBOXWRAPPER_DEFAULT_PATH = unicode("/usr/lib/gns3/vboxwrapper.py")
     else:
-        VBOXWRAPPER_DEFAULT_PATH = unicode("/usr/local/libexec/gns3/vboxwrapper.py")
+        VBOXWRAPPER_DEFAULT_PATH = unicode("/usr/local/libexec/gns3/vboxwrapper.py") # libexec is standard on BSD platforms
 
 # Default path to vboxwrapper working directory
 if os.environ.has_key("TEMP"):
@@ -92,6 +103,7 @@ else:
 
 Traditional_Capture_String = translate("Defaults", 'Wireshark Traditional Capture')
 Live_Traffic_Capture_String = translate("Defaults", 'Wireshark Live Traffic Capture')
+Pipe_Traffic_Capture_String = translate("Defaults", 'Wireshark Live Pipe Traffic Capture (experimental)')
 
 # Default predefined sets of Wireshark commands on various OSes:
 if platform.system() == 'Linux':
@@ -110,11 +122,14 @@ elif platform.system() == 'Windows' and os.path.exists("C:\Program Files (x86)\W
                            Traditional_Capture_String  + ' (Windows)': "C:\Program Files\Wireshark\wireshark.exe %c",
                            Live_Traffic_Capture_String + ' (Windows 64-bit)': 'tail.exe -f -c +0b %c | "C:\Program Files (x86)\Wireshark\wireshark.exe" -k -i -',
                            Live_Traffic_Capture_String + ' (Windows)': 'tail.exe -f -c +0b %c | "C:\Program Files\Wireshark\wireshark.exe" -k -i -',
+                           Pipe_Traffic_Capture_String + ' (Windows)': "C:\Program Files\Wireshark\wireshark.exe -k -i %p",
+                           Pipe_Traffic_Capture_String + ' (Windows 64-bit)': "C:\Program Files (x86)\Wireshark\wireshark.exe -k -i %p",
                            }
 elif platform.system() == 'Windows':
     CAPTURE_PRESET_CMDS = {
                            Traditional_Capture_String  + ' (Windows)': "C:\Program Files\Wireshark\wireshark.exe %c",
                            Live_Traffic_Capture_String + ' (Windows)': 'tail.exe -f -c +0b %c | "C:\Program Files\Wireshark\wireshark.exe" -k -i -',
+                           Pipe_Traffic_Capture_String + ' (Windows)': "C:\Program Files\Wireshark\wireshark.exe -k -i %p",
                            }
 elif platform.system() == 'Darwin':
     CAPTURE_PRESET_CMDS = {
@@ -165,46 +180,44 @@ if platform.system() == 'Linux' or platform.system().__contains__("BSD"):
                             }
 elif platform.system() == 'Windows'  and os.path.exists("C:\Program Files (x86)\\"):
     TERMINAL_PRESET_CMDS = {
-                            'Putty (Windows 64-bit)': '"C:\Program Files (x86)\\Putty\\putty.exe" -telnet %h %p',
-                            'Putty (Windows 32-bit)': '"C:\Program Files\\Putty\\putty.exe" -telnet %h %p',
-                            'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p',
-                            'SecureCRT (Windows 64-bit)': '"C:\Program Files (x86)\\VanDyke Software\SecureCRT\SecureCRT.EXE" /script "%s\securecrt.vbs"' % GNS3_RUN_PATH + ' /arg %d /T /telnet %h %p',
-                            'SecureCRT (Windows 32-bit)': '"C:\Program Files\\VanDyke Software\SecureCRT\SecureCRT.EXE" /script "%s\securecrt.vbs"' % GNS3_RUN_PATH + ' /arg %d /T /telnet %h %p',
-                            'TeraTerm (Windows 64-bit)': '"C:\Program Files (x86)\\teraterm\\ttermpro.exe" -telnet %h:%p',
-                            'TeraTerm (Windows 32-bit)': '"C:\Program Files\\teraterm\\ttermpro.exe" -telnet %h:%p',
-                            'Telnet (Windows)': 'start telnet %h %p',
+                            'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p -wt %d -gns3 5',
+                            'SuperPutty (Windows)': 'SuperPutty.exe -telnet "%h -P %p -wt \"%d\" -gns3 5 -skin 4"',
+                            'SecureCRT (Windows 64-bit)': '"C:\Program Files (x86)\\VanDyke Software\\SecureCRT\\SecureCRT.EXE" /SCRIPT securecrt.vbs /ARG %d /T /TELNET %h %p',
+                            'SecureCRT (Windows 32-bit)': '"C:\Program Files\\VanDyke Software\\SecureCRT\\SecureCRT.EXE" /SCRIPT securecrt.vbs /ARG %d /T /TELNET %h %p',
+                            'TeraTerm (Windows 64-bit)': '"C:\Program Files (x86)\\teraterm\\ttermpro.exe" /W=%d /T=1 %h %p',
+                            'TeraTerm (Windows 32-bit)': '"C:\Program Files\\teraterm\\ttermpro.exe" /W=%d /T=1 %h %p',
+                            'Telnet (Windows)': 'telnet %h %p',
                             }
 elif platform.system() == 'Windows':
     TERMINAL_PRESET_CMDS = {
-                            'Putty (Windows)': '"C:\Program Files\\Putty\\putty.exe" -telnet %h %p',
-                            'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p',
-                            'SecureCRT (Windows)': '"C:\Program Files\\VanDyke Software\SecureCRT\SecureCRT.EXE" /script "%s\securecrt.vbs"' % GNS3_RUN_PATH + ' /arg %d /T /telnet %h %p',
-                            'TeraTerm (Windows)': '"C:\Program Files\\teraterm\\ttermpro.exe" -telnet %h:%p',
-                            'Telnet (Windows)': 'start telnet %h %p'
+                            'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p -wt %d -gns3 5',
+                            'SuperPutty (Windows)': 'SuperPutty.exe -telnet "%h -P %p -wt \"%d\" -gns3 5 -skin 4"',
+                            'SecureCRT (Windows)': '"C:\Program Files\\VanDyke Software\\SecureCRT\\SecureCRT.EXE" /SCRIPT securecrt.vbs /ARG %d /T /TELNET %h %p',
+                            'TeraTerm (Windows)': '"C:\Program Files\\teraterm\\ttermpro.exe" /W=%d /T=1 %h %p',
+                            'Telnet (Windows)': 'telnet %h %p'
                             }
 elif platform.system() == 'Darwin':
     TERMINAL_PRESET_CMDS = {
                             'Terminal (Mac OS X)': "/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"telnet %h %p ; exit\"'",
                             'iTerm (Mac OS X)': "/usr/bin/osascript -e 'tell app \"iTerm\"' -e 'activate' -e 'set myterm to the first terminal' -e 'tell myterm' -e 'set mysession to (make new session at the end of sessions)' -e 'tell mysession' -e 'exec command \"telnet %h %p\"' -e 'set name to \"%d\"' -e 'end tell' -e 'end tell' -e 'end tell'",
-                            'SecureCRT (Mac OS X)': '/Applications/SecureCRT.app/Contents/MacOS/SecureCRT /arg %d /T /telnet %h %p'
+                            'SecureCRT (Mac OS X)': '/Applications/SecureCRT.app/Contents/MacOS/SecureCRT /ARG %d /T /TELNET %h %p'
                             }
 else:  # For unknown platforms, or if detection failed, we list all options.
     TERMINAL_PRESET_CMDS = {
-                            'Putty (Windows 64-bit)': '"C:\Program Files (x86)\\Putty\\putty.exe" -telnet %h %p',
-                            'Putty (Windows 32-bit)': '"C:\Program Files\\Putty\\putty.exe" -telnet %h %p',
-                            'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p',
-                            'SecureCRT (Windows 64-bit)': '"C:\Program Files (x86)\\VanDyke Software\SecureCRT\SecureCRT.EXE" /script "%s\securecrt.vbs"' % GNS3_RUN_PATH + ' /arg %d /T /telnet %h %p',
-                            'SecureCRT (Windows 32-bit)': '"C:\Program Files\\VanDyke Software\SecureCRT\SecureCRT.EXE" /script "%s\securecrt.vbs"' % GNS3_RUN_PATH + ' /arg %d /T /telnet %h %p',
-                            'TeraTerm (Windows 32-bit)': '"C:\Program Files\\teraterm\\ttermpro.exe" -telnet %h:%p',
-                            'TeraTerm (Windows 64-bit)': '"C:\Program Files (x86)\\teraterm\\ttermpro.exe" -telnet %h:%p',
-                            'Telnet (Windows)': 'start telnet %h %p',
+                            'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p -wt %d -gns3 5',
+                            'SuperPutty (Windows)': 'SuperPutty.exe -telnet "%h -P %p -wt \"%d\" -gns3 5 -skin 4"',
+                            'SecureCRT (Windows 64-bit)': '"C:\Program Files (x86)\\VanDyke Software\\SecureCRT\\SecureCRT.EXE" /SCRIPT securecrt.vbs /ARG %d /T /TELNET %h %p',
+                            'SecureCRT (Windows 32-bit)': '"C:\Program Files\\VanDyke Software\\SecureCRT\\SecureCRT.EXE" /SCRIPT securecrt.vbs /ARG %d /T /TELNET %h %p',
+                            'TeraTerm (Windows 32-bit)': '"C:\Program Files\\teraterm\\ttermpro.exe" /W=%d /T=1 %h %p',
+                            'TeraTerm (Windows 64-bit)': '"C:\Program Files (x86)\\teraterm\\ttermpro.exe" /W=%d /T=1 %h %p',
+                            'Telnet (Windows)': 'telnet %h %p',
                             'xterm (Linux/BSD)': 'xterm -T %d -e \'telnet %h %p\' >/dev/null 2>&1 &',
                             'Putty (Linux/BSD)': 'putty -telnet %h %p',
                             'Gnome Terminal (Linux/BSD)': 'gnome-terminal -t %d -e \'telnet %h %p\' >/dev/null 2>&1 &',
                             'KDE Konsole (Linux/BSD)': '/usr/bin/konsole --new-tab -p tabtitle=%d -e telnet %h %p >/dev/null 2>&1 &',
                             'Terminal (Mac OS X)': "/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"telnet %h %p ; exit\"'",
                             'iTerm (Mac OS X)': "/usr/bin/osascript -e 'tell app \"iTerm\"' -e 'activate' -e 'set myterm to the first terminal' -e 'tell myterm' -e 'set mysession to (make new session at the end of sessions)' -e 'tell mysession' -e 'exec command \"telnet %h %p\"' -e 'set name to \"%d\"' -e 'end tell' -e 'end tell' -e 'end tell'",
-                            'SecureCRT (Mac OS X)': '/Applications/SecureCRT.app/Contents/MacOS/SecureCRT /arg %d /T /telnet %h %p'
+                            'SecureCRT (Mac OS X)': '/Applications/SecureCRT.app/Contents/MacOS/SecureCRT /ARG %d /T /TELNET %h %p'
                             }
 
 # Default terminal command
@@ -215,6 +228,14 @@ elif sys.platform.startswith('win'):
 else:
     TERMINAL_DEFAULT_CMD = unicode(TERMINAL_PRESET_CMDS['xterm (Linux/BSD)'])
 
+# Default terminal serial command
+if sys.platform.startswith('win'):
+    TERMINAL_SERIAL_DEFAULT_CMD = unicode('putty.exe -serial %s -wt "%d [Local Console]" -gns3 5')
+elif sys.platform.startswith('darwin'):
+    TERMINAL_SERIAL_DEFAULT_CMD = unicode("/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"socat UNIX-CONNECT:%s stdio,raw,echo=0 ; exit\"'")
+else:
+    TERMINAL_SERIAL_DEFAULT_CMD = unicode('xterm -T %d -e \'socat UNIX-CONNECT:%s stdio,raw,echo=0\' > /dev/null 2>&1 &')
+
 # Default project directory
 if sys.platform.startswith('win') and os.environ.has_key("HOMEDRIVE") and os.environ.has_key("HOMEPATH"):
     PROJECT_DEFAULT_DIR = unicode(os.environ["HOMEDRIVE"] + os.environ["HOMEPATH"] + os.sep + 'GNS3' + os.sep + 'Projects', 'utf-8', errors='replace')
@@ -333,7 +354,11 @@ conf_vboxImage_defaults = {
     'nib_nb': 6,
     'nic': 'automatic',
     'guestcontrol_user': '',
-    'guestcontrol_password': ''
+    'guestcontrol_password': '',
+    'first_nic_managed': False,
+    'headless_mode': False,
+    'console_support': False,
+    'console_telnet_server': False,
 }
 
 conf_vboxImage_types = {
@@ -343,7 +368,11 @@ conf_vboxImage_types = {
     'nic_nb': int,
     'nic': str,
     'guestcontrol_user': str,
-    'guestcontrol_password': str
+    'guestcontrol_password': str,
+    'first_nic_managed': bool,
+    'headless_mode': bool,
+    'console_support': bool,
+    'console_telnet_server': bool,
 }
 
 conf_pixImage_defaults = {
@@ -448,14 +477,15 @@ conf_systemDynamips_defaults = {
     'workdir': '',
     'clean_workdir': True,
     'baseUDP': 10000,
-    'baseConsole': 2000,
-    'baseAUX': 2100,
+    'baseConsole': 2001,
+    'baseAUX': 2501,
     'ghosting': True,
     'jitsharing': False,
     'sparsemem': True,
     'mmap': True,
     'memory_limit': 512,
     'udp_incrementation': 100,
+    'detected_version': '',
     'import_use_HypervisorManager': True,
     'allocateHypervisorPerIOS': True,
     'HypervisorManager_binding': u'localhost',
@@ -475,6 +505,7 @@ conf_systemDynamips_types = {
     'mmap': bool,
     'memory_limit': int,
     'udp_incrementation': int,
+    'detected_version': unicode,
     'import_use_HypervisorManager': bool,
     'allocateHypervisorPerIOS': bool,
     'HypervisorManager_binding': unicode,
@@ -484,11 +515,13 @@ conf_systemGeneral_defaults = {
     'lang': 'en',
     'project_startup': True,
     'relative_paths': True,
-    'slow_start': 0,
+    'slow_start': 1,
     'autosave': 60,
     'term_cmd': '',
-    'use_shell': True,
+    'use_shell': False,
     'bring_console_to_front': False,
+    'term_serial_cmd': '',
+    'term_close_on_delete': False,
     'project_path': '.',
     'ios_path': '.',
     'status_points': True,
@@ -509,6 +542,8 @@ conf_systemGeneral_types = {
     'use_shell': bool,
     'bring_console_to_front': bool,
     'term_cmd': unicode,
+    'term_serial_cmd': unicode,
+    'term_close_on_delete': bool,
     'project_path': unicode,
     'ios_path': unicode,
     'status_points': bool,
@@ -544,8 +579,8 @@ conf_systemQemu_defaults = {
     'send_paths_external_Qemuwrapper': False,
     'QemuManager_binding': u'localhost',
     'qemuwrapper_port': 10525,
-    'qemuwrapper_baseUDP': 20000,
-    'qemuwrapper_baseConsole': 3000,
+    'qemuwrapper_baseUDP': 40000,
+    'qemuwrapper_baseConsole': 3001,
 }
 
 conf_systemQemu_types = {
@@ -568,6 +603,7 @@ conf_systemVBox_defaults = {
     'vboxwrapper_path':'',
     'vboxwrapper_workdir':'',
     'external_hosts':[],
+    'use_VBoxVmnames': True,
     'enable_VBoxWrapperAdvOptions' : False,
     'enable_VBoxAdvOptions' : False,
     'enable_GuestControl': False,
@@ -576,13 +612,14 @@ conf_systemVBox_defaults = {
     'VBoxManager_binding': u'localhost',
     'vboxwrapper_port': 11525,
     'vboxwrapper_baseUDP': 20900,
-    'vboxwrapper_baseConsole': 3900,
+    'vboxwrapper_baseConsole': 3501,
 }
 
 conf_systemVBox_types = {
     'vboxwrapper_path': unicode,
     'vboxwrapper_workdir': unicode,
     'external_hosts': list,
+    'use_VBoxVmnames': bool,
     'enable_VBoxWrapperAdvOptions' : bool,
     'enable_VBoxAdvOptions' : bool,
     'enable_GuestControl': bool,
