@@ -28,6 +28,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 debuglevel = 0
 
 import platform, os, sys
+import portTracker_lib as tracker
 
 if debuglevel > 0:
     if platform.system() == 'Windows':
@@ -57,9 +58,9 @@ def debugmsg(level, message):
 msg = "WELCOME to dynagen_vbox_lib.py"
 debugmsg(2, msg)
 
-from socket import socket, AF_INET, AF_INET6, SOCK_STREAM
 from dynamips_lib import NIO_udp, send, dowarning, debug, DynamipsError, validate_connect, Bridge, DynamipsVerError, get_reverse_udp_nio, Router, FRSW, ATMSW, ETHSW, DynamipsWarning
 import random
+import socket
 
 #version = "0.11.0.111211"
 (MAJOR, MINOR, SUB) = (0, 8, 2)
@@ -120,17 +121,11 @@ class VBox(object):
         self.host = name
 
         #connect to VBox Wrapper
-        if name.__contains__(':'):
-            # IPv6 address support
-            self.s = socket(AF_INET6, SOCK_STREAM)
-        else:
-            self.s = socket(AF_INET, SOCK_STREAM)
-        self.s.setblocking(0)
-        self.s.settimeout(300)
+        timeout = 60.0
         self._type = 'vboxwrapper'
         if not NOSEND:
             try:
-                self.s.connect((self.host, self.port))
+                self.s = socket.create_connection((self.host, self.port), timeout)
             except:
                 raise DynamipsError, '101-Could not connect to vboxwrapper at %s:%i' % (self.host, self.port)
         #version checking
@@ -169,7 +164,7 @@ class VBox(object):
         #all other needed variables
         self.name = name
         self.devices = []
-        self._baseconsole = 3900
+        self._baseconsole = 3501
         self.udp = 20900
         self.default_udp = self.udp
         self.starting_udp = self.udp
@@ -307,13 +302,21 @@ class AnyVBoxEmuDevice(object):
             'netcard': 'automatic',
             'guestcontrol_user' : None,
             'guestcontrol_password': None,
+            'first_nic_managed': False,
+            'headless_mode': False,
+            'console_support': False,
+            'console_telnet_server': False,
             }
-        #self._ram = 0
+
         self._nics = self.defaults['nics']
         self._netcard = self.defaults['netcard']
         self._capture = {}
         self._guestcontrol_user = self.defaults['guestcontrol_user']
         self._guestcontrol_password = self.defaults['guestcontrol_password']
+        self._first_nic_managed = self.defaults['first_nic_managed']
+        self._headless_mode = self.defaults['headless_mode']
+        self._console_support = self.defaults['console_support']
+        self._console_telnet_server = self.defaults['console_telnet_server']
 
         self.nios = {}
         for i in range(self._nics + 1):
@@ -321,14 +324,19 @@ class AnyVBoxEmuDevice(object):
 
         send(self.p, 'vbox create %s %s' % (self.vbox_dev_type, self.name))
         self.p.devices.append(self)
+
         #set the console to VBox baseconsole
-        self.console = self.p.baseconsole
+        self.track = tracker.portTracker()
+        self._console = self.track.allocateTcpPort(self.p.host, self.p.baseconsole)
+        send(self.p, 'vbox setattr %s console %i' % (self.name, self._console))
         self.p.baseconsole += 1
 
     def delete(self):
         """delete the virtualized device instance in VBox"""
         debugmsg(2, "AnyVBoxEmuDevice::delete()")
 
+        self.track.freeTcpPort(self.p.host, self.console)
+
         try:
             send(self.p, 'vbox delete %s' % self.name)
         except:
@@ -352,9 +360,9 @@ class AnyVBoxEmuDevice(object):
 
         if self.state == 'stopped':
             raise DynamipsWarning, 'virtualized device %s is already stopped' % self.name
-        r = send(self.p, 'vbox stop %s' % self.name)
-        r = send(self.p, 'vbox stop %s' % self.name)
         self.state = 'stopped'
+        r = send(self.p, 'vbox stop %s' % self.name)
+        r = send(self.p, 'vbox stop %s' % self.name)
         return r
 
     def reset(self):
@@ -450,8 +458,16 @@ class AnyVBoxEmuDevice(object):
 
         if type(console) != int or console < 1 or console > 65535:
             raise DynamipsError, 'invalid console port'
+        
+        if console == self._console:
+            return
+        
+        if not self.track.tcpPortIsFree(self.p.host, console):
+            raise DynamipsError, 'console port %i is already in use' % console
 
         send(self.p, 'vbox setattr %s console %i' % (self.name, console))
+        self.track.setTcpPort(self.p.host, console)
+        self.track.freeTcpPort(self.p.host, self._console)
         self._console = console
 
     def _getconsole(self):
@@ -579,6 +595,90 @@ class AnyVBoxEmuDevice(object):
 
     guestcontrol_password = property(_getguestcontrol_password, _setguestcontrol_password, doc='The VBox guestcontrol_password for this device')
 
+    def _setfirst_nic_managed(self, first_nic_managed):
+        """ Set if the first interface can be used by this virtualized device
+        first_nic_managed: (bool) first interface activation
+        """
+        debugmsg(2, "AnyVBoxEmuDevice::_setfirst_nic_managed(%s)" % str(first_nic_managed))
+
+        if type(first_nic_managed) != bool:
+            raise DynamipsError, 'invalid first nic managed option'
+
+        send(self.p, 'vbox setattr %s first_nic_managed %s' % (self.name, str(first_nic_managed)))
+        self._first_nic_managed = first_nic_managed
+
+    def _getfirst_nic_managed(self):
+        """ Returns the first_nic_managed option used by this virtualized device
+        """
+        debugmsg(3, "AnyVBoxEmuDevice::_getfirst_nic_managed(), returns %s" % str(self._first_nic_managed))
+
+        return self._first_nic_managed
+
+    first_nic_managed = property(_getfirst_nic_managed, _setfirst_nic_managed, doc='The first nic managed option by this virtualized device')
+    
+    def _setheadless_mode(self, headless_mode):
+        """ Set if the vm is to be started in headless mode
+        headless_mode: (bool) headless mode activation
+        """
+        debugmsg(2, "AnyVBoxEmuDevice::_setheadless_mode(%s)" % str(headless_mode))
+
+        if type(headless_mode) != bool:
+            raise DynamipsError, 'invalid headless mode option'
+
+        send(self.p, 'vbox setattr %s headless_mode %s' % (self.name, str(headless_mode)))
+        self._headless_mode = headless_mode
+
+    def _getheadless_mode(self):
+        """ Returns the headless mode option used by this virtualized device
+        """
+        debugmsg(3, "AnyVBoxEmuDevice::_getheadless_mode(), returns %s" % str(self._headless_mode))
+
+        return self._headless_mode
+
+    headless_mode = property(_getheadless_mode, _setheadless_mode, doc='The headless mode option by this virtualized device')
+    
+    def _setconsole_support(self, console_support):
+        """ Set if a console can be used to connect to the VM
+        console_support: (bool) console support activation
+        """
+        debugmsg(2, "AnyVBoxEmuDevice::_setconsole_support(%s)" % str(console_support))
+
+        if type(console_support) != bool:
+            raise DynamipsError, 'invalid console support option'
+
+        send(self.p, 'vbox setattr %s console_support %s' % (self.name, str(console_support)))
+        self._console_support = console_support
+
+    def _getconsole_support(self):
+        """ Returns the console support option used by this virtualized device
+        """
+        debugmsg(3, "AnyVBoxEmuDevice::_getconsole_support(), returns %s" % str(self._console_support))
+
+        return self._console_support
+
+    console_support = property(_getconsole_support, _setconsole_support, doc='The console support option by this virtualized device')
+    
+    def _setconsole_telnet_server(self, console_telnet_server):
+        """ Set if a telnet server to access console via Telnet
+        console_telnet_server: (bool) telnet server activation
+        """
+        debugmsg(2, "AnyVBoxEmuDevice::_setconsole_telnet_server(%s)" % str(console_telnet_server))
+
+        if type(console_telnet_server) != bool:
+            raise DynamipsError, 'invalid console telnet server support option'
+
+        send(self.p, 'vbox setattr %s console_telnet_server %s' % (self.name, str(console_telnet_server)))
+        self._console_telnet_server = console_telnet_server
+
+    def _getconsole_telnet_server(self):
+        """ Returns the console telnet server support option used by this virtualized device
+        """
+        debugmsg(3, "AnyVBoxEmuDevice::_getconsole_telnet_server(), returns %s" % str(self._console_telnet_server))
+
+        return self._console_telnet_server
+
+    console_telnet_server = property(_getconsole_telnet_server, _setconsole_telnet_server, doc='The console telnet server support option by this virtualized device')
+
     def capture(self, interface, path):
         """ Set the capture file path for a specific interface
         interface: (int) interface number
@@ -813,10 +913,12 @@ class AnyVBoxEmuDevice(object):
         from qemu_lib import AnyEmuDevice
         # hide vbox internal interface (self._nics - 1)
         slot_info = '   Slot 0 hardware is ' + self._netcard + ' with ' + str(self._nics) + ' Ethernet interfaces\n'
-        slot_info = slot_info + "      Ethernet1 is the VirtualBox management interface\n"
+        ignore_ports = [0] # port 0 doesn't exists
+        if not self._first_nic_managed:
+            slot_info = slot_info + "      Ethernet1 is the VirtualBox management interface\n"
+            ignore_ports.append(1) # port 1 is the VirtualBox management interface
         for port in self.nios:
-            if port in [0,1]:
-                # port 0 doesn't exists and port 1 is the VirtualBox management interface
+            if port in ignore_ports:
                 continue
             slot_info = slot_info + "      Ethernet" + str(port)
             if self.nios[port] != None:
@@ -850,12 +952,18 @@ class AnyVBoxEmuDevice(object):
 
         info = '\n'.join([
             '%s %s is %s' % (self._ufd_machine, self.name, self.state),
-            #'  Hardware is %s %s with %s MB RAM' % (self._ufd_hardware, self.model_string, self._ram),
             '  Hardware is %s %s' % (self._ufd_hardware, self.model_string),
             '  %s\'s wrapper runs on %s:%s' % (self._ufd_machine, self.dynamips.host, self.dynamips.port),
             '  Image is %s' % self.image
-            #'  %s KB NVRAM, %s MB flash size' % (self.nvram, self.disk0)
         ])
+        
+        if self.console_support and self.console_telnet_server:
+            info += '\n' + '  Listenning on console port %s (access via Telnet)' % self.console
+        elif self.console_support:
+            info += '\n' + '  Console support is activated (local access only)'
+
+        if self.headless_mode:
+            info += '\n' + '  Headless mode is activated'
 
         if hasattr(self, 'extended_info'):
             info += '\n' + self.extended_info()
@@ -878,7 +986,7 @@ class VBoxDevice(AnyVBoxEmuDevice):
     basehostname = 'VBOX'
     _ufd_machine = 'VirtualBox guest'
     _ufd_hardware = 'VirtualBox Virtualized System'
-    available_options = ['image', 'nics', 'netcard', 'guestcontrol_user', 'guestcontrol_password']
+    available_options = ['image', 'nics', 'netcard', 'guestcontrol_user', 'guestcontrol_password', 'first_nic_managed', 'headless_mode', 'console_support', 'console_telnet_server']
 
 def nosend_vbox(flag):
     """ If true, don't actually send any commands to the back end.
