@@ -29,6 +29,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 debuglevel = 0
 
 import platform, os
+import portTracker_lib as tracker
 
 if debuglevel > 0:
     if platform.system() == 'Windows':
@@ -58,15 +59,15 @@ def debugmsg(level, message):
 msg = "WELCOME to qemu_lib.py"
 debugmsg(2, msg)
 
-from socket import socket, AF_INET, AF_INET6, SOCK_STREAM
 from dynamips_lib import NIO_udp, send, dowarning, debug, DynamipsError, validate_connect, Bridge, DynamipsVerError, get_reverse_udp_nio, Router, FRSW, ATMSW, ETHSW, DynamipsWarning
 import random
 import hashlib
+import socket
 
 #version = "0.11.0.091411"
 (MAJOR, MINOR, SUB, RCVER) = (0, 2, 1, .1)
 INTVER = MAJOR * 10000 + MINOR * 100 + SUB + RCVER
-STRVER = '0.2.2-RC1'
+STRVER = '0.8.3'
 NOSEND = False  # Disable sending any commands to the back end for debugging
 
 class UDPConnection:
@@ -122,17 +123,11 @@ class Qemu(object):
         self.host = name
 
         #connect to Qemu Wrapper
-        if name.__contains__(':'):
-            # IPv6 address support
-            self.s = socket(AF_INET6, SOCK_STREAM)
-        else:
-            self.s = socket(AF_INET, SOCK_STREAM)
-        self.s.setblocking(0)
-        self.s.settimeout(300)
+        timeout = 60.0
         self._type = 'qemuwrapper'
         if not NOSEND:
             try:
-                self.s.connect((self.host, self.port))
+                self.s = socket.create_connection((self.host, self.port), timeout)
             except:
                 raise DynamipsError, 'Could not connect to qemuwrapper at %s:%i' % (self.host, self.port)
         #version checking
@@ -162,8 +157,8 @@ class Qemu(object):
         #all other needed variables
         self.name = name
         self.devices = []
-        self._baseconsole = 3000
-        self.udp = 20000
+        self._baseconsole = 3001
+        self.udp = 40000
         self.default_udp = self.udp
         self.starting_udp = self.udp
         self._workingdir = None
@@ -173,7 +168,6 @@ class Qemu(object):
 
     def close(self):
         """ Close the connection to the Qemuwrapper (but leave it running)"""
-        debugmsg(2, "Qemu::close(%s, %s)" % (unicode(name), str(port)))
 
         self.s.close()
 
@@ -228,7 +222,7 @@ class Qemu(object):
         if type(qemupath) not in [str, unicode]:
             raise DynamipsError, 'invalid Qemu path'
         # send to qemuwrapper encased in quotes to protect spaces
-        send(self, 'qemuwrapper qemu_path %s' % '"' + qemupath + '"')
+        send(self, 'qemuwrapper qemu_path %s' % '"' + qemupath.replace('\\', '/') + '"')
         self._qemupath = qemupath
 
     def _getqemupath(self):
@@ -249,7 +243,7 @@ class Qemu(object):
         if type(qemuimgpath) not in [str, unicode]:
             raise DynamipsError, 'invalid Qemu-img path'
         # send to qemuwrapper encased in quotes to protect spaces
-        send(self, 'qemuwrapper qemu_img_path %s' % '"' + qemuimgpath + '"')
+        send(self, 'qemuwrapper qemu_img_path %s' % '"' + qemuimgpath.replace('\\', '/') + '"')
         self._qemuimgpath = qemuimgpath
 
     def _getqemuimgpath(self):
@@ -273,7 +267,7 @@ class Qemu(object):
         if type(directory) not in [str, unicode]:
             raise DynamipsError, 'invalid directory'
         # send to qemuwrapper encased in quotes to protect spaces
-        send(self, 'qemuwrapper working_dir %s' % '"' + directory + '"')
+        send(self, 'qemuwrapper working_dir %s' % '"' + directory.replace('\\', '/') + '"')
         self._workingdir = directory
 
     def _getworkingdir(self):
@@ -351,12 +345,16 @@ class AnyEmuDevice(object):
         send(self.p, 'qemu create %s %s' % (self.qemu_dev_type, self.name))
         self.p.devices.append(self)
         #set the console to Qemu baseconsole
-        self.console = self.p.baseconsole
+        self.track = tracker.portTracker()
+        self._console = self.track.allocateTcpPort(self.p.host, self.p.baseconsole)
+        send(self.p, 'qemu setattr %s console %i' % (self.name, self._console))
         self.p.baseconsole += 1
 
     def delete(self):
         """delete the emulated device instance in Qemu"""
         debugmsg(2, "AnyEmuDevice::delete()")
+        
+        self.track.freeTcpPort(self.p.host, self.console)
 
         try:
             send(self.p, 'qemu delete %s' % self.name)
@@ -380,8 +378,9 @@ class AnyEmuDevice(object):
 
         if self.state == 'stopped':
             raise DynamipsWarning, 'emulated device %s is already stopped' % self.name
-        r = send(self.p, 'qemu stop %s' % self.name)
+
         self.state = 'stopped'
+        r = send(self.p, 'qemu stop %s' % self.name)
         return r
 
     def clean(self):
@@ -419,7 +418,15 @@ class AnyEmuDevice(object):
         if type(console) != int or console < 1 or console > 65535:
             raise DynamipsError, 'invalid console port'
 
+        if console == self._console:
+            return
+
+        if not self.track.tcpPortIsFree(self.p.host, console):
+            raise DynamipsError, 'console port %i is already in use' % console
+
         send(self.p, 'qemu setattr %s console %i' % (self.name, console))
+        self.track.setTcpPort(self.p.host, console)
+        self.track.freeTcpPort(self.p.host, self._console)
         self._console = console
 
     def _getconsole(self):
@@ -532,7 +539,7 @@ class AnyEmuDevice(object):
             raise DynamipsError, 'invalid options'
 
         #send the options enclosed in quotes to protect them
-        send(self.p, 'qemu setattr %s options %s' % (self.name, '"' + options + '"'))
+        send(self.p, 'qemu setattr %s options %s' % (self.name, '"' + options.replace('"', '\\"') + '"'))
         self._options = options
 
     def _getoptions(self):
@@ -555,7 +562,7 @@ class AnyEmuDevice(object):
 
         # Can't verify existance of image because path is relative to backend
         #send the image filename enclosed in quotes to protect it
-        send(self.p, 'qemu setattr %s image %s' % (self.name, '"' + image + '"'))
+        send(self.p, 'qemu setattr %s image %s' % (self.name, '"' + image.replace('\\', '/') + '"'))
         self._image = image
 
     def _getimage(self):
@@ -578,7 +585,7 @@ class AnyEmuDevice(object):
             send(self.p, 'qemu delete_capture %s %i' % (self.name, interface))
             del self._capture[interface]
         else:
-            send(self.p, 'qemu create_capture %s %i %s' % (self.name, interface, path))
+            send(self.p, 'qemu create_capture %s %i %s' % (self.name, interface, '"' + path.replace('\\', '/') + '"'))
             self._capture[interface] = path
 
     def idleprop(self,prop):
@@ -845,7 +852,7 @@ class IDS(AnyEmuDevice):
 
         # Can't verify existance of image because path is relative to backend
         #send the image filename enclosed in quotes to protect it
-        send(self.p, 'qemu setattr %s image1 %s' % (self.name, '"' + image + '"'))
+        send(self.p, 'qemu setattr %s image1 %s' % (self.name, '"' + image.replace('\\', '/') + '"'))
         self._image1 = image
 
     def _getimage1(self):
@@ -866,7 +873,7 @@ class IDS(AnyEmuDevice):
 
         # Can't verify existance of image because path is relative to backend
         #send the image filename enclosed in quotes to protect it
-        send(self.p, 'qemu setattr %s image2 %s' % (self.name, '"' + image + '"'))
+        send(self.p, 'qemu setattr %s image2 %s' % (self.name, '"' + image.replace('\\', '/') + '"'))
         self._image2 = image
 
     def _getimage2(self):
@@ -918,7 +925,7 @@ class ASA(AnyEmuDevice):
 
         # Can't verify existance of image because path is relative to backend
         #send the initrd filename enclosed in quotes to protect it
-        send(self.p, 'qemu setattr %s initrd %s' % (self.name, '"' + initrd + '"'))
+        send(self.p, 'qemu setattr %s initrd %s' % (self.name, '"' + initrd.replace('\\', '/') + '"'))
         self._initrd = initrd
 
     def _getinitrd(self):
@@ -939,7 +946,7 @@ class ASA(AnyEmuDevice):
 
         # Can't verify existance of image because path is relative to backend
         #send the kernel filename enclosed in quotes to protect it
-        send(self.p, 'qemu setattr %s kernel %s' % (self.name, '"' + kernel + '"'))
+        send(self.p, 'qemu setattr %s kernel %s' % (self.name, '"' + kernel.replace('\\', '/') + '"'))
         self._kernel = kernel
 
     def _getkernel(self):
@@ -959,7 +966,7 @@ class ASA(AnyEmuDevice):
             raise DynamipsError, 'invalid kernel command line'
 
         #send the kernel command line enclosed in quotes to protect it
-        send(self.p, 'qemu setattr %s kernel_cmdline %s' % (self.name, '"' + kernel_cmdline + '"'))
+        send(self.p, 'qemu setattr %s kernel_cmdline %s' % (self.name, '"' + kernel_cmdline.replace('"', '\\"') + '"'))
         self._kernel_cmdline = kernel_cmdline
 
     def _getkernel_cmdline(self):
