@@ -22,11 +22,11 @@
 #debuglevel: 0=disabled, 1=default, 2=debug, 3=deep debug
 debuglevel = 0
 
-import sys, os, re
+import sys, os, re, socket
 import GNS3.Globals as globals
 from GNS3.Dynagen.dynagen import Dynagen, DEVICETUPLE
 from GNS3.Utils import translate, debug, getWindowsInterfaces
-from PyQt4 import QtCore, QtGui
+from PyQt4 import QtCore, QtGui, QtNetwork
 
 def debugmsg(level, message):
     if debuglevel == 0:
@@ -43,6 +43,8 @@ class DynagenSub(Dynagen):
         Dynagen.__init__(self)
         self.gns3_data = None
         self.rpcap_mapping = {}
+        self.local_addresses = map(lambda addr: unicode(addr.toString()), QtNetwork.QNetworkInterface.allAddresses())
+        self.local_addresses += ['0.0.0.0', '127.0.0.1', 'localhost', '::1', '0:0:0:0:0:0:0:1', QtNetwork.QHostInfo.localHostName()]
 
     def check_replace_GUID_NIO(self, filename):
         """ Check and replace non-existing GUID (network interface ID) on Windows
@@ -51,13 +53,13 @@ class DynagenSub(Dynagen):
 
         file = open(filename,'r')
         lines = file.readlines()
-        cregex = re.compile("^.*nio_gen_eth:(.*)")
+        cregex = re.compile(r"""^.*nio_gen_eth:(\\device\\npf_{[a-fA-F0-9\-]*}).*""")
         niolist = []
 
         for currentline in lines:
             currentline = currentline.lower().strip()
             match_obj = cregex.match(currentline)
-            if match_obj:
+            if match_obj and match_obj.group(1) not in niolist:
                 niolist.append(match_obj.group(1))
 
         self.rpcap_mapping.clear()
@@ -67,11 +69,21 @@ class DynagenSub(Dynagen):
             rpcaps = getWindowsInterfaces()
             interfaces = {}
             for rpcap in rpcaps:
-                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*})\ :\ (.*)""", rpcap)
+                match = re.search(r"""^rpcap://(\\Device\\NPF_{[a-fA-F0-9\-]*})(.*)""", rpcap)
                 interface_guid = str(match.group(1)).lower()
                 interfaces[interface_guid] = unicode(match.group(2)).strip()
-                name_match = re.search(r"""^(.*)\ on local host:.*""", match.group(2))
-                self.rpcap_mapping['nio_gen_eth:' + interface_guid] = name_match.group(1)
+
+                name_match = re.search(r"""^\ :.*:\ (.+)""", match.group(2))
+                if name_match:
+                    interface_name = name_match.group(1)
+                else:
+                    # The interface name could not be found, let's use the interface model instead
+                    model_match = re.search(r"""^\ :\ (.*)\ on local host:.*""", match.group(2))
+                    if model_match:
+                        interface_name = model_match.group(1)
+                    else:
+                        interface_name = translate("DynagenSub", "Unknown name")
+                self.rpcap_mapping['nio_gen_eth:' + interface_guid] = interface_name
 
             for nio in niolist:
                 if not interfaces.has_key(nio):
@@ -163,11 +175,10 @@ class DynagenSub(Dynagen):
                 if ':' in host:
                     # unpack the server and port
                     # controlPort is ignored
-                    #(host, controlPort) = host.split(':')  # What if we provide IPv6 ???
                     controlPort = host.split(':')[-1]
                     host = self.getHost(host)
-                if emulator == 'qemu' and (host == globals.GApp.systconf['qemu'].QemuManager_binding or host == 'localhost') and globals.GApp.systconf['qemu'].enable_QemuManager:
-                    globals.GApp.QemuManager.startQemu(int(controlPort))
+                if emulator == 'qemu' and globals.GApp.systconf['qemu'].enable_QemuManager and globals.GApp.systconf['qemu'].import_use_QemuManager and host in self.local_addresses:
+                    globals.GApp.QemuManager.startQemu(int(controlPort), host)
                     debugmsg(2, "DynagenSub::open_config(), entered QemuManager")
 
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
@@ -284,12 +295,11 @@ class DynagenSub(Dynagen):
                                 print translate("DynagenSub", "Local PIX image %s cannot be found, use image %s instead") \
                                 % (unicode(device['image']), image_name)
                                 device['image'] = image_name
-                elif emulator == 'vbox' and (host == globals.GApp.systconf['vbox'].VBoxManager_binding or host == 'localhost') and globals.GApp.systconf['vbox'].enable_VBoxManager:
+                elif emulator == 'vbox' and globals.GApp.systconf['vbox'].enable_VBoxManager and globals.GApp.systconf['vbox'].import_use_VBoxManager and host in self.local_addresses:
                     debugmsg(2, "DynagenSub::open_config(), entered VBoxManager")
-                    globals.GApp.VBoxManager.startVBox(int(controlPort))
+                    globals.GApp.VBoxManager.startVBox(int(controlPort), host)
 
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
-                    #"""
                     if server['workingdir']:
                         abspath = os.path.join(os.path.dirname(FILENAME), unicode(server['workingdir']))
                         debugmsg(3, "DynagenSub::open_config(), 'vbox', abspath = %s" % unicode(abspath))
@@ -304,22 +314,6 @@ class DynagenSub(Dynagen):
                     for subsection in server.sections:
                         device = server[subsection]
                         debugmsg(3, "DynagenSub::open_config(), 'vbox', device...")
-                        #debugmsg(3, "DynagenSub::open_config(), device = %s" % str(device))
-                        """  # Disabled whole test, as it fails on multi-host systems, if VBoxManager enabled.
-
-                        # Check if the image path is a relative path
-                        if os.path.exists(device['image']) == False:
-                            abspath = os.path.join(os.path.dirname(FILENAME), unicode(device['image']))
-                            debugmsg(3, "DynagenSub::open_config(), 'vbox', abspath = %s" % str(abspath))
-                            if os.path.exists(abspath):
-                                device['image'] = abspath
-                        #"""
-                        #if device.name == 'VBoxDevice' and device['image']:
-                        #    debugmsg(2, "DynagenSub.py:  if device.name == 'VBoxDevice' and device['image']")
-                        #    #Warning about non-existend VBox Machine disabled.
-                        #    #I decided to skip check on import. Just imported VMs won't run, so user can manually fix them.
-                        #    #VirtualBox will give correct error anyway, on VM start attempt.
-                    #"""
             else:
                 server.host = server.name
                 debugmsg(3, "DynagenSub::open_config(), server.host = %s, server.name = %s" % (unicode(server.host), unicode(server.name)))
@@ -334,8 +328,8 @@ class DynagenSub(Dynagen):
                     controlPort = '7200'
 
                 # need to start local hypervisors
-                if (server.host == globals.GApp.systconf['dynamips'].HypervisorManager_binding or server.host == 'localhost') and \
-                    globals.GApp.HypervisorManager and globals.GApp.systconf['dynamips'].import_use_HypervisorManager:
+
+                if globals.GApp.HypervisorManager and globals.GApp.systconf['dynamips'].import_use_HypervisorManager and server.host in self.local_addresses:
                     debugmsg(2, "DynagenSub::open_config(), entered Dynamips HypervisorManager")
 
                     # update server.host and server.name to match with Hypervisor Manager Binding configuration,
@@ -346,8 +340,16 @@ class DynagenSub(Dynagen):
 #                        server.name = server.host + ':' + controlPort
 
                     debug("Start hypervisor on port: " + str(controlPort))
-                    hypervisor = globals.GApp.HypervisorManager.startNewHypervisor(int(controlPort), processcheck=False)
-                    globals.GApp.HypervisorManager.waitHypervisor(hypervisor)
+
+                    try:
+                        s = socket.create_connection((server.host, int(controlPort)), 60.0)
+                        s.close()
+                        print "Warning: a process is already running on port %i, please consider closing it or killing it as this may negatively impact your topology" % int(controlPort)
+                    except:
+                        pass
+
+                    hypervisor = globals.GApp.HypervisorManager.startNewHypervisor(int(controlPort), binding=server.host, processcheck=False)
+                    globals.GApp.HypervisorManager.waitHypervisor(hypervisor, binding=server.host)
 
                     # Check if this is a relative working directory path and convert to an absolute path if necessary
                     if server['workingdir']:
@@ -393,15 +395,14 @@ class DynagenSub(Dynagen):
                                 if len(selected_images):
                                     message = unicode("Local IOS image %s\ncannot be found for hypervisor %s:%s\n\nPlease choose an alternative image:") % (device['image'], server.host, controlPort)
                                     selected_images.sort()
-                                    (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "IOS image"),
+                                    (selection, ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("DynagenSub", "IOS image"),
                                                                                       translate("DynagenSub", message), selected_images, 0, False)
                                     if ok:
                                         image_to_use = unicode(selection)
 
                                 if image_to_use == None and len(selected_images) == 0:
-                                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, 'DynagenSub',
-                                                               translate("IOS image", "IOS image %s cannot be found for hypervisor %s:%s and cannot find an alternative %s image")
-                                                                % (device['image'], server.host, controlPort, device.name))
+                                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("DynagenSub", "IOS image"),
+                                                               translate("DynagenSub", "IOS image %s cannot be found and cannot find an alternative %s image") % (device['image'], device.name))
                                     continue
                                 if image_to_use == None and len(selected_images) > 1:
                                     for image in selected_images:
@@ -414,11 +415,13 @@ class DynagenSub(Dynagen):
                                 image_name = globals.GApp.iosimages[image_to_use].filename
                                 ram = globals.GApp.iosimages[image_to_use].default_ram
                                 idlepc = globals.GApp.iosimages[image_to_use].idlepc
-                                print translate("DynagenSub", "Local IOS image %s cannot be found for hypervisor %s:s, use image %s instead") \
-                                % (device['image'], server.host, controlPort, image_name)
+                                print translate("DynagenSub", "Local IOS image %s cannot be found, use image %s instead") % (device['image'], image_name)
                                 device['image'] = image_name
                                 device['ram'] = ram
-                                device['idlepc'] = idlepc
+                                if idlepc:
+                                    device['idlepc'] = idlepc
+                                else:
+                                    print translate("DynagenSub", "WARNING: no IDLE PC value found for %s, please apply one before using this topology!") % device['image']
 
                         # check if the config file is accessible, if not find an alternative config
                         elif device.has_key('cnfg') and device['cnfg']:
@@ -444,11 +447,14 @@ class DynagenSub(Dynagen):
                                         basename = ntpath.basename(device['cnfg'])
 
                                     new_config_path = globals.GApp.workspace.projectConfigs + os.sep + basename
-
-                                    print translate("DynagenSub", "Local configuration %s cannot be found for router %s, use configuration %s instead") \
-                                    % (device['cnfg'], device.name, new_config_path)
+                                    if not os.path.exists(new_config_path):
+                                        # this case is handled in NETFile
+                                        print translate("DynagenSub", "Local configuration %s cannot be found for router %s, use default base configuration instead") \
+                                        % (device['cnfg'], device.name)
+                                    else:
+                                        print translate("DynagenSub", "Local configuration %s cannot be found for router %s, use configuration %s instead") \
+                                        % (device['cnfg'], device.name, new_config_path)
                                     device['cnfg'] = new_config_path
-
                 current += 1
 
         progress.setValue(count)
