@@ -191,7 +191,7 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
                     else:
                         extracted_ios = path + '.image'
                     reply = QtGui.QMessageBox.question(self, translate("IOSDialog", "IOS Image"),
-                                                       translate("IOSDialog", "The IOS image is compressed. Would you like to uncompress it? Yes will create %s") % os.path.basename(extracted_ios),
+                                                       translate("IOSDialog", "The IOS image is compressed. Would you like to uncompress it?\nThis will create %s") % os.path.basename(extracted_ios),
                                                        QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
                     if reply == QtGui.QMessageBox.Yes:
                         if os.path.exists(extracted_ios):
@@ -487,7 +487,8 @@ class IOSDialog(QtGui.QDialog, Ui_IOSDialog):
             conf.baseConsole = self.spinBoxBaseConsole.value()
             conf.baseAUX = self.spinBoxBaseAUX.value()
             self.spinBoxBaseUDP.setValue(conf.baseUDP + 100)
-            self.spinBoxBaseConsole.setValue(conf.baseConsole + 10)
+            self.spinBoxBaseConsole.setValue(conf.baseConsole + 20)
+            self.spinBoxBaseAUX.setValue(conf.baseAUX + 20)
             globals.GApp.hypervisors[hypervisorkey] = conf
             self.treeWidgetHypervisor.resizeColumnToContents(0)
             self.treeWidgetHypervisor.resizeColumnToContents(1)
