@@ -44,6 +44,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
 
         self.dynagen = globals.GApp.dynagen
         self.default_symbol = True
+        self.consoleProcesses = []
 
         # status used in the topology summary
         self.state = 'stopped'
@@ -76,6 +77,35 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
 
         self.setZValue(1)
 
+    def __del__(self):
+
+        if globals.GApp.systconf['general'].term_close_on_delete:
+            # closing terminal programs
+            for console in self.consoleProcesses:
+                console.poll()
+                if console.returncode == None:
+                    # the process hasn't returned yet (still active)
+                    debug("Sending terminate signal to terminal program (pid %i)" % console.pid)
+                    try:
+                        console.terminate()
+                    except:
+                        debug("Error while sending the signal to terminal program (pid %i)" % console.pid)
+                        continue
+        self.consoleProcesses = []
+        
+    def clearClosedConsoles(self):
+        """ Refresh the list of opened terminal programs.
+        """
+
+        updated_list = []
+        for console in self.consoleProcesses:
+            console.poll()
+            if console.returncode == None:
+                # the process hasn't returned yet (still active)
+                updated_list.append(console)
+        self.consoleProcesses = updated_list
+        debug("%s has %i terminal program(s) connected to itself" % (self.hostname, len(self.consoleProcesses)))
+
     def setRenderers(self, render_normal, render_select):
         """ renderer_normal: QtSvg.QSvgRenderer
             renderer_select: QtSvg.QSvgRenderer
@@ -116,7 +146,7 @@ class AbstractNode(QtSvg.QGraphicsSvgItem):
                                           self.hostname)
         if ok and text:
             text = unicode(text)
-            if not re.search(r"""^[\w,.]*$""", text, re.UNICODE):
+            if not re.search(r"""^[\w,.-\[\]]*$""", text, re.UNICODE):
                 QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "Hostname"),
                                            translate("AbstractNode", "Please use only alphanumeric characters"))
                 self.changeHostname()
