@@ -80,6 +80,7 @@ class AnyEmuDevice(AbstractNode, AnyEmuDefaults):
     def __del__(self):
 
         self.delete_emudev()
+        AbstractNode.__del__(self)
 
     def delete_emudev(self, delete_persistent=False):
         """ Delete this emulated device
@@ -87,7 +88,8 @@ class AnyEmuDevice(AbstractNode, AnyEmuDefaults):
         if self.emudev:
             try:
                 self.stopNode()
-                del self.dynagen.devices[self.hostname]
+                if self.dynagen.devices.has_key(self.hostname):
+                    del self.dynagen.devices[self.hostname]
                 if self.emudev in self.qemu.devices:
                     self.qemu.devices.remove(self.emudev)
                 self.dynagen.update_running_config()
@@ -330,10 +332,10 @@ class AnyEmuDevice(AbstractNode, AnyEmuDefaults):
             except:
                 if progress:
                     raise
-
-            self.shutdownInterfaces()
-            self.state = self.emudev.state
-            globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.emudev.state)
+            finally:
+                self.shutdownInterfaces()
+                self.state = self.emudev.state
+                globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.emudev.state)
 
     def suspendNode(self, progress=False):
         """ Suspend this node
@@ -355,7 +357,10 @@ class AnyEmuDevice(AbstractNode, AnyEmuDefaults):
         """
 
         if self.emudev and self.emudev.state == 'running' and self.emudev.console:
-            console.connect(self.emudev.dynamips.host, self.emudev.console, self.hostname)
+            proc = console.connect(self.emudev.dynamips.host, self.emudev.console, self.hostname)
+            if proc:
+                self.consoleProcesses.append(proc)
+        AbstractNode.clearClosedConsoles(self)
 
     def isStarted(self):
         """ Returns True if this device is started
@@ -373,6 +378,15 @@ class AnyEmuDevice(AbstractNode, AnyEmuDefaults):
 
         AbstractNode.mousePressEvent(self, event)
 
+    def changeConsolePort(self):
+        """ Called to change the console port
+        """
+
+        if self.emudev.state != 'stopped':
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AnyEmuDevice", "Console"), translate("AnyEmuDevice", "Cannot change the console port while the node is running"))
+            return
+        AbstractNode.changeConsolePort(self)
+
 class PIX(AnyEmuDevice, PIXDefaults):
     instance_counter = 0
     model = '525'
