@@ -72,11 +72,12 @@ class ETHSW(AbstractNode):
         if self.ethsw:
             try:
                 self.ethsw.delete()
-                del self.dynagen.devices[self.hostname]
+                if self.dynagen.devices.has_key(self.hostname):
+                    del self.dynagen.devices[self.hostname]
                 if self.ethsw in self.hypervisor.devices:
                     self.hypervisor.devices.remove(self.ethsw)
                 self.dynagen.update_running_config()
-            except lib.DynamipsErrorHandled:
+            except:
                 pass
             self.ethsw = None
 
@@ -155,11 +156,14 @@ class ETHSW(AbstractNode):
         """
 
         if len(self.config['ports']) == len(self.getConnectedInterfaceList()):
-            port = max(self.config['ports']) + 1
+            if len(self.config['ports']) == 0:
+                port = 1
+                self.config['vlans'][1] = []
+            else:
+                port = max(self.config['ports']) + 1
             self.config['ports'][port] = 'access'
             self.config['vlans'][1].append(port)
 
-
     def getInterfaces(self):
         """ Returns all interfaces
         """
