@@ -126,6 +126,7 @@ class IOSRouter(AbstractNode):
     def __del__(self):
 
         self.delete_router()
+        AbstractNode.__del__(self)
 
     def delete_router(self):
         """ Delete this router
@@ -136,7 +137,8 @@ class IOSRouter(AbstractNode):
                 self.stopNode()
                 # don't forget to delete this router in Dynamips
                 self.router.delete()
-                del self.dynagen.devices[self.hostname]
+                if self.dynagen.devices.has_key(self.hostname):
+                    del self.dynagen.devices[self.hostname]
                 if self.router in self.hypervisor.devices:
                     self.hypervisor.devices.remove(self.router)
                 self.dynagen.update_running_config()
@@ -607,10 +609,11 @@ class IOSRouter(AbstractNode):
                 self.router.start()
             if self.router.state == 'suspended':
                 self.router.resume()
-        except:
+        except lib.DynamipsErrorHandled:
             if progress:
                 raise
             else:
+                print translate("IOSRouter", "Cannot start router %s: lost communication with server %s:%s") % (self.router.name, self.router.dynamips.host, str(self.router.dynamips.port))
                 return
         self.startupInterfaces()
         self.state = self.router.state
@@ -624,13 +627,16 @@ class IOSRouter(AbstractNode):
         if self.router.state != 'stopped':
             try:
                 self.router.stop()
-            except:
+            except lib.DynamipsErrorHandled:
                 if progress:
                     raise
-            self.shutdownInterfaces()
-            self.state = self.router.state
-            self.updateToolTips()
-            globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.router.state)
+                else:
+                    print translate("IOSRouter", "Cannot stop router %s: lost communication with server %s:%s") % (self.router.name, self.router.dynamips.host, str(self.router.dynamips.port))
+            finally:
+                self.shutdownInterfaces()
+                self.state = self.router.state
+                self.updateToolTips()
+                globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.router.state)
 
     def reloadNode(self, progress=False):
         """ Reload this node
@@ -639,7 +645,7 @@ class IOSRouter(AbstractNode):
         if self.router.state != 'running':
             return
         self.stopNode(progress)
-        time.sleep(1)
+        time.sleep(0.2)
         self.startNode(progress)
 
     def suspendNode(self, progress=False):
@@ -649,31 +655,56 @@ class IOSRouter(AbstractNode):
         if self.router.state == 'running':
             try:
                 self.router.suspend()
-            except:
+            except lib.DynamipsErrorHandled:
                 if progress:
                     raise
-            self.suspendInterfaces()
-            self.state = self.router.state
-            self.updateToolTips()
-            globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.router.state)
+                else:
+                    print translate("IOSRouter", "Cannot suspend router %s: lost communication with server %s:%s") % (self.router.name, self.router.dynamips.host, str(self.router.dynamips.port))
+            finally:
+                self.suspendInterfaces()
+                self.state = self.router.state
+                self.updateToolTips()
+                globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(self.hostname, self.router.state)
 
     def console(self):
         """ Start a telnet console and connect it to this router
         """
 
         if self.router and self.router.state == 'running' and self.router.console:
-            console.connect(self.hypervisor.host, self.router.console, self.hostname)
+            proc = console.connect(self.hypervisor.host, self.router.console, self.hostname)
+            if proc:
+                self.consoleProcesses.append(proc)
+        AbstractNode.clearClosedConsoles(self)
 
     def aux(self):
         """ Start a telnet console and connect it to this router's AUX port
         """
         if not self.router.aux:
-            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("AbstractNode", "AUX port"),
+            QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("IOSRouter", "AUX port"),
                                        translate("AbstractNode", "AUX port not available for this router model or base AUX port is set to 0 in preferences"))
             return False
 
         if self.router and self.router.state == 'running':
-            console.connect(self.hypervisor.host, self.router.aux, self.hostname)
+            proc = console.connect(self.hypervisor.host, self.router.aux, self.hostname)
+            if proc:
+                self.consoleProcesses.append(proc)
+        AbstractNode.clearClosedConsoles(self)
+
+    def changeConsolePort(self):
+        """ Called to change the console port
+        """
+
+        if self.router.state != 'stopped':
+            QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("IOSRouter", "Console"), translate("IOSRouter", "You must restart this router after changing its console port"))
+        AbstractNode.changeConsolePort(self)
+
+    def changeAUXPort(self):
+        """ Called to change the aux port
+        """
+
+        if self.router.state != 'stopped':
+            QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("IOSRouter", "AUX port"), translate("IOSRouter", "You must restart this router after changing its AUX port"))
+        AbstractNode.changeAUXPort(self)
 
     def isStarted(self):
         """ Returns True if this router is started
