@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# code@gns3.net
+# http://www.gns3.net/contact
 #
 
 import os
@@ -79,7 +79,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
                 item.setText(0, name)
                 item.setIcon(0,  QtGui.QIcon(':/symbols/' + symbol))
                 item.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(':/symbols/' + symbol))
-        
+
         # load libraries
         for (library_name, conf) in globals.GApp.libraries.iteritems():
             self.addLibrarySymbols(library_name, conf.path)
@@ -90,7 +90,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
     def slotAdd(self):
         """ Add a symbol to the node list
         """
-    
+
         symbols = self.treeWidgetSymbols.selectedItems()
         for symbol in symbols:
             name = unicode(symbol.text(0))
@@ -104,7 +104,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
             SYMBOLS.append(
                            {'name': name, 'object': DecorativeNode,
                             'normal_svg_file': normal_svg_file,
-                            'select_svg_file': selected_svg_file, 
+                            'select_svg_file': selected_svg_file,
                             'translated': False })
 
             item = QtGui.QTreeWidgetItem(self.treeWidgetNodes)
@@ -113,10 +113,17 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
             item.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(translate("nodesDock", "Decorative node")))
             item.setSelected(True)
 
+            self.lineEditNodeName.setText(name)
+            index = self.comboBoxNodeType.findText(translate("nodesDock", "Decorative node"))
+            if index != -1:
+                self.comboBoxNodeType.setCurrentIndex(index)
+            self.treeWidgetNodes.setCurrentItem(item)
+
+
     def slotRemove(self):
         """ Remove a symbol from the node list
         """
-        
+
         nodes = self.treeWidgetNodes.selectedItems()
         for node in nodes:
             name = unicode(node.text(0))
@@ -135,7 +142,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
         current = self.treeWidgetNodes.currentItem()
         if current and self.lineEditNodeName.text():
             name = unicode(self.lineEditNodeName.text())
-            type = str(self.comboBoxNodeType.currentText())
+            type = unicode(self.comboBoxNodeType.currentText())
             for symbol in SYMBOLS:
                 if symbol['name'] == unicode(current.text(0)):
                     symbol['name'] = name
@@ -146,7 +153,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
                     break
             current.setText(0, name)
             current.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(type))
-                
+
     def slotNodeSelectionChanged(self):
         """ Check if an entry is selected in the list of nodes
         """
@@ -156,7 +163,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
             self.pushButtonRemove.setEnabled(True)
             self.treeWidgetSymbols.clearSelection()
 
-            
+
     def slotSymbolSelectionChanged(self):
         """ Check if an entry is selected in the list of symbols
         """
@@ -174,14 +181,14 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
         type_name = node.data(0, QtCore.Qt.UserRole).toString()
         index = self.comboBoxNodeType.findText(type_name)
         if index != -1:
-            self.comboBoxNodeType.setCurrentIndex(index) 
+            self.comboBoxNodeType.setCurrentIndex(index)
         else:
             print 'Warning: cannot find type ' + type_name
 
     def slotLibrarySelected(self, item):
         """ Load library settings
         """
-        
+
         if item and not item.parent():
             path = item.data(0, QtCore.Qt.UserRole).toString()
             self.lineEditLibrary.setText(path)
@@ -189,7 +196,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
     def addLibrarySymbols(self, library_name, path):
         """ Add library symbols in the list
         """
-    
+
         library = QtGui.QTreeWidgetItem()
         library.setText(0, library_name)
         library.setIcon(0,  QtGui.QIcon(':/icons/package.svg'))
@@ -201,22 +208,22 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
             item.setText(0, symbol)
             item.setIcon(0, QtGui.QIcon(':/' + library_name + '/' + symbol))
             item.setData(0, QtCore.Qt.UserRole, QtCore.QVariant(':/' + library_name + '/' + symbol))
-            
+
     def slotAddLibrary(self):
         """ Add a library
         """
-        
+
         path = unicode(self.lineEditLibrary.text())
         if not path:
             return
         library_name = os.path.basename(path)
         if len(self.treeWidgetSymbols.findItems(library_name, QtCore.Qt.MatchFixedString)):
-            QtGui.QMessageBox.critical(self, translate("SymbolManager", "Library"), unicode(translate("SymbolManager", "This library is already loaded: %s")) % library_name)
+            QtGui.QMessageBox.critical(self, translate("SymbolManager", "Library"), translate("SymbolManager", "This library is already loaded: %s") % library_name)
             return
         if not QtCore.QResource.registerResource(path, ":/" + library_name):
-            QtGui.QMessageBox.critical(self, translate("SymbolManager", "Library"), unicode(translate("SymbolManager", "Can't open library: %s")) % path)
+            QtGui.QMessageBox.critical(self, translate("SymbolManager", "Library"), translate("SymbolManager", "Can't open library: %s") % path)
             return
-            
+
         self.addLibrarySymbols(library_name, path)
         conf = libraryConf()
         conf.path = path
@@ -231,7 +238,7 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
             return
         library_name = os.path.basename(path)
         if not QtCore.QResource.unregisterResource(path, ":/" + library_name):
-            QtGui.QMessageBox.critical(self, translate("SymbolManagement", "Library"), unicode(translate("SymbolManager", "Can't remove library: %s")) % path)
+            QtGui.QMessageBox.critical(self, translate("SymbolManagement", "Library"), translate("SymbolManager", "Can't remove library: %s") % path)
             return
 
         library = self.treeWidgetSymbols.findItems(library_name, QtCore.Qt.MatchFixedString)[0]
@@ -241,11 +248,11 @@ class SymbolManager(QtGui.QDialog, Ui_SymbolManager):
     def slotCallLibraryBrowser(self):
         """ Call a file system browser to select a library
         """
-        fb = fileBrowser(translate('SymbolManagement', 'Library path'), parent=self)
+        fb = fileBrowser(translate('SymbolManagement', 'Library path'), directory=globals.GApp.systconf['general'].project_path, parent=self)
         (path, selected) = fb.getFile()
 
         if path is not None and path != '':
             self.lineEditLibrary.clear()
             self.lineEditLibrary.setText(os.path.normpath(path))
-            
+
 
