@@ -19,8 +19,7 @@
 # http://www.gns3.net/contact
 #
 
-
-import os, glob, socket, sys, base64, time
+import os, glob, socket, sys, base64, re
 import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Dynagen.qemu_lib as qlib
 import GNS3.Dynagen.dynagen_vbox_lib as vboxlib
@@ -101,6 +100,8 @@ class Topology(QtGui.QGraphicsScene):
         self.dynagen.ghostsizes.clear()
         self.dynagen.bridges.clear()
         self.dynagen.autostart.clear()
+        self.dynagen.running_config.clear()
+        self.dynagen.defaults_config.clear()
 
         for item in self.items():
             self.removeItem(item)
@@ -128,13 +129,13 @@ class Topology(QtGui.QGraphicsScene):
         # Clear Undo Stack first
         self.undoStack.clear()
         globals.interfaceLabels.clear()
-
         for n_key in self.__nodes.copy().iterkeys():
             self.deleteNode(n_key)
         self.__nodes = {}
         while len(self.__links) > 0:
-            o = self.__links.pop()
-            self.removeItem(o)
+            link = self.__links.pop()
+            link.stopCapturing(showMessage=False, refresh=False)
+            self.removeItem(link)
         self.__links = set()
         self.node_baseid = 0
         self.link_baseid = 0
@@ -207,14 +208,14 @@ class Topology(QtGui.QGraphicsScene):
             external_hypervisor_key = selected_hypervisor
         else:
             external_hypervisor_key = hypervisors[0]
-        globals.GApp.hypervisors[external_hypervisor_key].used_ram += node.default_ram
-        (host, port) = external_hypervisor_key.rsplit(':',  1)
 
+        (host, port) = external_hypervisor_key.rsplit(':',  1)
         if self.dynagen.dynamips.has_key(external_hypervisor_key):
             debug("Use an external hypervisor: " + external_hypervisor_key)
             dynamips_hypervisor = self.dynagen.dynamips[external_hypervisor_key]
         else:
             debug("Connection to an external hypervisor: " + external_hypervisor_key)
+            globals.GApp.hypervisors[external_hypervisor_key].used_ram += node.default_ram
             hypervisor_conf = globals.GApp.hypervisors[external_hypervisor_key]
             # use project workdir in priority
             if globals.GApp.workspace.projectWorkdir:
@@ -242,6 +243,7 @@ class Topology(QtGui.QGraphicsScene):
         """ Apply IOS base config
         """
 
+        debug("Applying IOS base config %s" % config_path)
         if not os.access(config_path, os.F_OK):
             QtGui.QMessageBox.warning(globals.GApp.mainWindow, translate("Topology", "IOS Base config"), translate("Topology", "The base config file (%s) specified for this IOS can not be found. Your router will start with a blank configuration.") % config_path)
             return
@@ -281,8 +283,13 @@ class Topology(QtGui.QGraphicsScene):
             debug("Disable mmap")
             node.set_string_option('mmap', False)
         if globals.GApp.systconf['dynamips'].sparsemem:
-            debug("Enable sparse memory")
-            node.set_string_option('sparsemem', True)
+            if globals.GApp.HypervisorManager and image_conf.platform in ('c2600', 'c1700'):
+                # Workaround: sparse memory feature is not activated on c2600 and c1700 platforms because Dynamips freezes on console message 
+                # "Press ENTER to get the prompt" after a restart. (Bug is inside Dynamips).
+                debug("Do not enable sparse memory for this platform (known bug workaround)")
+            else:
+                debug("Enable sparse memory")
+                node.set_string_option('sparsemem', True)
         if globals.GApp.systconf['dynamips'].ghosting:
             debug("Enable Ghost IOS")
             node.set_ghostios(True)
@@ -591,12 +598,20 @@ class Topology(QtGui.QGraphicsScene):
                     return False
 
                 devices = []
-                for name in globals.GApp.vboximages.keys():
-                    devices.append(name)
+                current_vboximages = []
+                for device in self.__nodes.itervalues():
+                    if isinstance(device, VBoxDevice):
+                        current_vboximages.append(device.get_config()['image'])
+                for (name, conf) in globals.GApp.vboximages.iteritems():
+                    if not conf.filename in current_vboximages:
+                        devices.append(name)
+                if len(devices) == 0:
+                    QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "VirtualBox guest"), translate("Topology", "All configured VMs already in use. You may add or clone additional VMs in VirtualBox"))
+                    return False
 
                 if node.image_reference:
                     conf = globals.GApp.vboximages[node.image_reference]
-                elif len(globals.GApp.vboximages) > 1:
+                elif len(devices) > 1:
 
                     devices.sort()
                     (selection,  ok) = QtGui.QInputDialog.getItem(globals.GApp.mainWindow, translate("Topology", "VirtualBox guest"),
@@ -615,6 +630,11 @@ class Topology(QtGui.QGraphicsScene):
 
                 vmname = conf.filename  #Qemu's Disk Image equals to VMname/UUID in this release.
 
+                for device in self.__nodes.itervalues():
+                    if isinstance(device, VBoxDevice) and device.get_config()['image'] == vmname:
+                        QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("Topology", "VirtualBox guest"), translate("Topology", "VM already used, please clone your VM in VirtualBox"))
+                        return False
+
                 if self.vboxDeviceSetup(node, vmname) == False:
                     init_vbox_emu_id(node.id)
                     return False
@@ -625,6 +645,10 @@ class Topology(QtGui.QGraphicsScene):
                 node.set_string_option('netcard', conf.nic)
                 node.set_string_option('guestcontrol_user', conf.guestcontrol_user)
                 node.set_string_option('guestcontrol_password', conf.guestcontrol_password)
+                node.set_string_option('first_nic_managed', conf.first_nic_managed)
+                node.set_string_option('headless_mode', conf.headless_mode)
+                node.set_string_option('console_support', conf.console_support)
+                node.set_string_option('console_telnet_server', conf.console_telnet_server)
 
             if isinstance(node, JunOS):
 
@@ -848,6 +872,25 @@ class Topology(QtGui.QGraphicsScene):
             if iosConfig:
                 self.applyIOSBaseConfig(node, iosConfig)
 
+            #FIXME: ugly temporary workaround to have VBox VMs with same hostname as in VirtualBox names (this is not a clean solution as params are sent twice to vboxwrapper)
+            if isinstance(node, VBoxDevice) and globals.GApp.systconf['vbox'].use_VBoxVmnames:
+                image = node.config['image'].strip()
+                vmname = image
+                for (name, conf) in globals.GApp.vboximages.iteritems():
+                    if conf.filename == image:
+                        vmname = name
+                # white spaces have to be replaced
+                p = re.compile('\s+', re.UNICODE)
+                vmname = p.sub("_", vmname)
+                if re.search(r"""^[\w,.-\[\]]*$""", vmname, re.UNICODE):
+                    node.reconfigNode(vmname)
+                    if node.hostnameDiplayed():
+                        # force to redisplay the hostname
+                        node.removeHostname()
+                        node.showHostname()
+                else:
+                    print translate("Topology", "Couldn't set the same hostname as in VirtualBox for %s because non alphanumeric characters have been detected") % node.hostname
+
         except (lib.DynamipsVerError, lib.DynamipsError), msg:
             if isinstance(node, IOSRouter):
                 # check if dynamips can create its files
@@ -908,7 +951,7 @@ class Topology(QtGui.QGraphicsScene):
                     # internal hypervisor
                     image_conf = globals.GApp.iosimages[globals.GApp.systconf['dynamips'].HypervisorManager_binding + ':' + router.image]
                     if globals.GApp.HypervisorManager and len(image_conf.hypervisors) == 0:
-                        globals.GApp.HypervisorManager.unallocateHypervisor(node, router.dynamips.port)
+                        globals.GApp.HypervisorManager.unallocateHypervisor(node, router.dynamips.host ,router.dynamips.port)
                 else:
                     # external hypevisor
                     external_hypervisor_key = router.dynamips.host + ':' + str(router.dynamips.port)
