@@ -20,15 +20,16 @@
 #
 
 import sys, os
+import subprocess as sub
+import GNS3.Globals as globals
+import GNS3.Dynagen.portTracker_lib as tracker
 from PyQt4 import QtGui, QtCore, QtNetwork
 from GNS3.Ui.ConfigurationPages.Form_PreferencesDynamips import Ui_PreferencesDynamips
 from GNS3.Config.Objects import systemDynamipsConf
 from GNS3.HypervisorManager import HypervisorManager
 from GNS3.Config.Config import ConfDB
-from GNS3.Utils import fileBrowser, translate, testOpenFile
+from GNS3.Utils import fileBrowser, translate, testOpenFile, testIfWritableDir
 from GNS3.Config.Defaults import DYNAMIPS_DEFAULT_PATH, DYNAMIPS_DEFAULT_WORKDIR
-import GNS3.Globals as globals
-import subprocess as sub
 
 class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
 
@@ -194,13 +195,16 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
 
         if path:
             self.dynamips_workdir.setText(os.path.normpath(path))
-            
+
             if sys.platform.startswith('win'):
                 try:
                     path.encode('ascii')
                 except:
                     QtGui.QMessageBox.warning(globals.preferencesWindow, translate("UiConfig_PreferencesDynamips", "Working directory"), translate("UiConfig_PreferencesDynamips", "The path you have selected should contains only ascii (English) characters. Dynamips (Cygwin DLL) doesn't support unicode on Windows!"))
 
+            if not testIfWritableDir(path):
+                QtGui.QMessageBox.critical(globals.preferencesWindow, translate("UiConfig_PreferencesDynamips", "Working directory"), translate("UiConfig_PreferencesDynamips", "Dynamips working directory must be writable!"))
+
     def __testDynamips(self):
         
         if len(globals.GApp.topology.nodes):
@@ -216,7 +220,7 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
                 return
 
         try:
-            p = sub.Popen([globals.GApp.systconf['dynamips'].path, '--help'], stdout = sub.PIPE)
+            p = sub.Popen([globals.GApp.systconf['dynamips'].path, '--help'], stdout=sub.PIPE)
             dynamips_stdout = p.communicate()
         except OSError:
             self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Failed to start Dynamips")  + '</font>')
@@ -231,15 +235,39 @@ class UiConfig_PreferencesDynamips(QtGui.QWidget, Ui_PreferencesDynamips):
             version_2nd = int(version_raw.split('.')[1])
             version_3rd = int(version_raw.split('.')[2].split('-')[0])
             dynamips_ver = str(version_1st)+'.'+str(version_2nd)+'.'+str(version_3rd)+'-'+version_raw.split('.')[2].split('-')[1]
+            globals.GApp.systconf['dynamips'].detected_version = unicode(dynamips_ver)
+            ConfDB().sync()
         except:
             self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Failed to determine version of Dynamips.")  + '</font>')
-            return            
-        
+            return
+
         if version_2nd < 2 or version_3rd < 8:
             self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Found Dynamips %s, which is not supported. Use 0.2.8+ instead.") % dynamips_ver + '</font>')
             return
 
-        self.saveConf()
+        if not testIfWritableDir(globals.GApp.systconf['dynamips'].workdir):
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Dynamips working directory does not exist or is not writable")  + '</font>')
+            return
+
+        track = tracker.portTracker()
+        # Check if any of the first 10 ports are free to use by hypervisors
+        not_free_ports = track.getNotAvailableTcpPortRange(globals.GApp.systconf['dynamips'].HypervisorManager_binding, globals.hypervisor_baseport, 10)
+        if len(not_free_ports):
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Ports already in use %s<br>Please choose another base port value<br>or close these ports" % str(not_free_ports))  + '</font>')
+            return
+
+        # Check if any of the first 10 ports are free to use for dynamips console ports
+        not_free_ports = track.getNotAvailableTcpPortRange(globals.GApp.systconf['dynamips'].HypervisorManager_binding, globals.GApp.systconf['dynamips'].baseConsole, 10)
+        if len(not_free_ports):
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Ports already in use %s<br>Please choose another base console value<br>or close these ports" % str(not_free_ports))  + '</font>')
+            return
+
+        # Check if any of the first 10 ports are free to use for dynamips aux ports
+        not_free_ports = track.getNotAvailableTcpPortRange(globals.GApp.systconf['dynamips'].HypervisorManager_binding, globals.GApp.systconf['dynamips'].baseAUX, 10)
+        if len(not_free_ports):
+            self.labelDynamipsStatus.setText('<font color="red">' + translate("UiConfig_PreferencesDynamips", "Ports already in use %s<br>Please choose another base AUX value<br>or close these ports" % str(not_free_ports))  + '</font>')
+            return
+
         if globals.GApp.systconf['dynamips'].path:
             globals.GApp.workspace.clear()
             globals.GApp.HypervisorManager = HypervisorManager()
