@@ -22,11 +22,12 @@
 import sys, os, re, platform
 import GNS3.Globals as globals
 import subprocess
+import GNS3.Dynagen.portTracker_lib as tracker
 from PyQt4 import QtGui, QtCore, QtNetwork
 from GNS3.QemuManager import QemuManager
 from GNS3.Ui.ConfigurationPages.Form_PreferencesQemu import Ui_PreferencesQemu
 from GNS3.Config.Objects import systemQemuConf, qemuImageConf, pixImageConf, junosImageConf, asaImageConf, idsImageConf
-from GNS3.Utils import fileBrowser, translate
+from GNS3.Utils import fileBrowser, translate, testIfWritableDir
 from GNS3.Config.Defaults import QEMUWRAPPER_DEFAULT_PATH, QEMUWRAPPER_DEFAULT_WORKDIR
 from GNS3.Config.Config import ConfDB
 
@@ -317,11 +318,15 @@ class UiConfig_PreferencesQemu(QtGui.QWidget, Ui_PreferencesQemu):
         """ Get a working directory for Qemuwrapper from the file system
         """
 
-        fb = fileBrowser(translate('UiConfig_PreferencesQemu', 'Local Qemu working directory'), parent=globals.preferencesWindow)
+        fb = fileBrowser(translate('Page_PreferencesQemu', 'Local Qemu working directory'), parent=globals.preferencesWindow)
         path = fb.getDir()
 
         if path:
-            self.lineEditQemuwrapperWorkdir.setText(os.path.normpath(path))
+            path = os.path.normpath(path)
+            self.lineEditQemuwrapperWorkdir.setText(path)
+
+            if not testIfWritableDir(path):
+                QtGui.QMessageBox.critical(globals.preferencesWindow, translate("Page_PreferencesQemu", "Capture directory"), translate("Page_PreferencesQemu", "Qemu working directory must be writable!"))
 
     def slotSelectQemuPath(self):
         """ Get a path to Qemu from the file system
@@ -870,6 +875,10 @@ class UiConfig_PreferencesQemu(QtGui.QWidget, Ui_PreferencesQemu):
                 self.labelQemuStatus.setText('<font color="red">' + translate("UiConfig_PreferencesQemu", "Qemuwrapper path doesn't exist")  + '</font>')
                 return
 
+            if globals.GApp.systconf['qemu'].enable_QemuManager and not testIfWritableDir(globals.GApp.systconf['qemu'].qemuwrapper_workdir):
+                self.labelQemuStatus.setText('<font color="red">' + translate("UiConfig_PreferencesQemu", "Qemu working directory does not exist or is not writable")  + '</font>')
+                return
+
             globals.GApp.workspace.clear()
             globals.GApp.QemuManager = QemuManager()
             if globals.GApp.QemuManager.preloadQemuwrapper() == False:
@@ -891,7 +900,7 @@ class UiConfig_PreferencesQemu(QtGui.QWidget, Ui_PreferencesQemu):
                 # Qemu-0.11 for Windows is buggy, and fails to return 'qemu --help' results.
                 qemu_check= 0
                 try:
-                    p = subprocess.Popen([globals.GApp.systconf['qemu'].qemu_path, '--help'], cwd=globals.GApp.systconf['qemu'].qemuwrapper_workdir, stdout = subprocess.PIPE)
+                    p = subprocess.Popen([globals.GApp.systconf['qemu'].qemu_path, '-help'], cwd=globals.GApp.systconf['qemu'].qemuwrapper_workdir, stdout = subprocess.PIPE)
                     qemustdout = p.communicate()
                 except:
                     self.labelQemuStatus.setText('<font color="red">' + translate("UiConfig_PreferencesQemu", "Failed to start qemu")  + '</font>')
@@ -926,6 +935,14 @@ class UiConfig_PreferencesQemu(QtGui.QWidget, Ui_PreferencesQemu):
             except OSError:
                 bPEMUfound = False
 
+            if globals.GApp.systconf['qemu'].enable_QemuManager:
+                track = tracker.portTracker()
+                # Check if any of the first 10 ports are free to use for Qemu device console ports
+                not_free_ports = track.getNotAvailableTcpPortRange(globals.GApp.systconf['qemu'].QemuManager_binding, globals.GApp.systconf['qemu'].qemuwrapper_baseConsole, 10)
+                if len(not_free_ports):
+                    self.labelQemuStatus.setText('<font color="red">' + translate("UiConfig_PreferencesQemu", "Ports already in use %s<br>Please choose another base console value<br>or close these ports" % str(not_free_ports))  + '</font>')
+                    return
+
             try:
                 p = subprocess.Popen([globals.GApp.systconf['qemu'].qemu_img_path], cwd=globals.GApp.systconf['qemu'].qemuwrapper_workdir)
                 p.terminate()
