@@ -29,7 +29,7 @@ from PyQt4 import QtGui, QtCore, QtNetwork
 from GNS3.VBoxManager import VBoxManager
 from GNS3.Ui.ConfigurationPages.Form_PreferencesVirtualBox import Ui_PreferencesVirtualBox
 from GNS3.Config.Objects import systemVBoxConf, vboxImageConf
-from GNS3.Utils import fileBrowser, translate
+from GNS3.Utils import fileBrowser, translate, testIfWritableDir
 from GNS3.Config.Defaults import VBOXWRAPPER_DEFAULT_PATH, VBOXWRAPPER_DEFAULT_WORKDIR
 from GNS3.Config.Config import ConfDB
 
@@ -69,16 +69,13 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         self.connect(self.treeWidgetVBoxImages,  QtCore.SIGNAL('itemSelectionChanged()'),  self.slotVBoxImageSelectionChanged)
 
         # Auto-fill of VirtualBox VM name
-        self.connect(self.comboBoxNameVBoxImage, QtCore.SIGNAL('editTextChanged(QString)'), self.VBoxImage, QtCore.SLOT('setText(QString)'))
+        self.connect(self.comboBoxNameVBoxImage, QtCore.SIGNAL('editTextChanged(QString)'), self.VBoxID, QtCore.SLOT('setText(QString)'))
 
         # Refresh VM list
         self.connect(self.pushButtonRefresh, QtCore.SIGNAL('clicked()'), self.slotRefreshVMlist)
 
-        # Hide base console port - not available in VirtualBox:
-        self.label_30.setVisible(False)
-        self.baseConsole.setVisible(False)
-
         self.loadConf()
+        self.comboBoxNameVBoxImage.addItem("")
 
     def slotRefreshVMlist(self):
 
@@ -89,11 +86,11 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         self.comboBoxNameVBoxImage.clear()
         self.comboBoxNameVBoxImage.addItem("")
 
-
         if not vbox:
             if globals.GApp.systconf['vbox'].enable_VBoxManager:
                 host = globals.GApp.systconf['vbox'].VBoxManager_binding
                 port = globals.GApp.systconf['vbox'].vboxwrapper_port
+
                 if globals.GApp.VBoxManager.startVBox(port) == False:
                     return
             else:
@@ -172,7 +169,7 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
             self.label_10.setVisible(True)
             # GuestControl
             self.VBoxcheckBoxEnableGuestControl.setVisible(True)
-            self.label_8.setVisible(True)
+            #self.label_8.setVisible(True)
             self.label_4.setVisible(True)
             self.label_7.setVisible(True)
             self.VBoxGuestControl_User.setVisible(True)
@@ -183,7 +180,7 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
             self.label_10.setVisible(False)
             # GuestControl
             self.VBoxcheckBoxEnableGuestControl.setVisible(False)
-            self.label_8.setVisible(False)
+            #self.label_8.setVisible(False)
             self.label_4.setVisible(False)
             self.label_7.setVisible(False)
             self.VBoxGuestControl_User.setVisible(False)
@@ -227,6 +224,11 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         self.comboBoxExternalVBoxwrappers.addItems(self.conf.external_hosts)
         self.external_hosts = self.conf.external_hosts
 
+        if self.conf.use_VBoxVmnames:
+            self.checkBoxUseVBoxVMNames.setCheckState(QtCore.Qt.Checked)
+        else:
+            self.checkBoxUseVBoxVMNames.setCheckState(QtCore.Qt.Unchecked)
+
         if self.conf.enable_VBoxManager:
             self.checkBoxEnableVBoxManager.setCheckState(QtCore.Qt.Checked)
         else:
@@ -265,7 +267,7 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         self.port.setValue(self.conf.vboxwrapper_port)
         self.baseUDP.setValue(self.conf.vboxwrapper_baseUDP)
         self.baseConsole.setValue(self.conf.vboxwrapper_baseConsole)
-        
+
         # VirtualBox settings
         for (name, conf) in globals.GApp.vboximages.iteritems():
 
@@ -276,7 +278,7 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
             item.setText(1, conf.filename)
             
         self.treeWidgetVBoxImages.resizeColumnToContents(0)
-        self.treeWidgetVBoxImages.sortItems(0, QtCore.Qt.AscendingOrder) # Sort accoroding to GNS3 name
+        self.treeWidgetVBoxImages.sortItems(1, QtCore.Qt.AscendingOrder) # Sort according to GNS3 name
 
     def saveConf(self):
 
@@ -285,7 +287,11 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         self.conf.vboxwrapper_workdir = unicode(self.lineEditVBoxwrapperWorkdir.text(), 'utf-8', errors='replace')
         self.conf.external_hosts = self.external_hosts
         self.conf.VBoxManager_binding = unicode(self.comboBoxBinding.currentText(), 'utf-8', errors='replace')
-        
+
+        if self.checkBoxUseVBoxVMNames.checkState() == QtCore.Qt.Checked:
+            self.conf.use_VBoxVmnames = True
+        else:
+            self.conf.use_VBoxVmnames = False
         if self.checkBoxEnableVBoxManager.checkState() == QtCore.Qt.Checked:
             self.conf.enable_VBoxManager = True
         else:
@@ -345,19 +351,22 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         path = fb.getDir()
 
         if path:
-            self.lineEditVBoxwrapperWorkdir.setText(os.path.normpath(path))
+            path = os.path.normpath(path)
+            self.lineEditVBoxwrapperWorkdir.setText(path)
+
+            if not testIfWritableDir(path):
+                QtGui.QMessageBox.critical(globals.preferencesWindow, translate("UiConfig_PreferencesVirtualBox", "Working directory"), translate("UiConfig_PreferencesVirtualBox", "Vbox working directory must be writable!"))
                         
     def slotSaveVBoxImage(self):
         """ Add/Save VBox Image in the list of VBox images
         """
 
-        #name = unicode(self.comboBoxNameVBoxImage.text())
-        name = unicode(self.comboBoxNameVBoxImage.currentText(), 'utf-8', errors='replace')
-        image = unicode(self.VBoxImage.text(), 'utf-8', errors='replace')
+        name = unicode(self.VBoxID.text(), 'utf-8', errors='replace')
+        image = unicode(self.comboBoxNameVBoxImage.currentText(), 'utf-8', errors='replace')
         
         if not name or not image:
             QtGui.QMessageBox.critical(globals.preferencesWindow, translate("Page_PreferencesVirtualBox", "VirtualBox guest"), 
-                                       translate("Page_PreferencesVirtualBox", "Identifier and binary image must be set!"))
+                                       translate("Page_PreferencesVirtualBox", "Identifier and VM must be set!"))
             return
 
         if globals.GApp.vboximages.has_key(name):
@@ -365,6 +374,14 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
             item_to_update = self.treeWidgetVBoxImages.findItems(name, QtCore.Qt.MatchFixedString)[0]
             item_to_update.setText(1, image)
         else:
+            
+            # white spaces have to be replaced
+            p = re.compile('\s+', re.UNICODE)
+            name = p.sub("_", name)
+            if not re.search(r"""^[\w,.-\[\]]*$""", name, re.UNICODE):
+                QtGui.QMessageBox.critical(globals.preferencesWindow, translate("Page_PreferencesVirtualBox", "VirtualBox guest"), 
+                                           translate("Page_PreferencesVirtualBox", "Identifier name must contains only alphanumeric characters!"))
+                return
             # else create a new entry
             item = QtGui.QTreeWidgetItem(self.treeWidgetVBoxImages)
             # image name column
@@ -384,12 +401,32 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
         conf.filename = image
         conf.nic_nb = self.VBoxNICNb.value()
         conf.nic = str(self.VBoxNIC.currentText())
+        if self.checkBoxVBoxFirstInterfaceManaged.checkState() == QtCore.Qt.Checked:
+            conf.first_nic_managed = True
+        else:
+            conf.first_nic_managed = False
+
+        if self.checkBoxVboxConsoleSupport.checkState() == QtCore.Qt.Checked:
+            conf.console_support = True
+        else:
+            conf.console_support = False
+
+        if self.checkBoxVBoxHeadlessMode.checkState() == QtCore.Qt.Checked:
+            conf.headless_mode = True
+        else:
+            conf.headless_mode = False
+
+        if self.checkBoxVboxConsoleServer.checkState() == QtCore.Qt.Checked:
+            conf.console_telnet_server = True
+        else:
+            conf.console_telnet_server = False
+
         conf.guestcontrol_user = str(self.VBoxGuestControl_User.text())
         conf.guestcontrol_password = str(self.VBoxGuestControl_Password.text())
-        
+
         globals.GApp.vboximages[name] = conf
         self.treeWidgetVBoxImages.resizeColumnToContents(0)
-    
+
     def slotDeleteVBoxImage(self):
         """ Delete VBox Image from the list of VBox images
         """
@@ -400,7 +437,7 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
             name = unicode(item.text(0), 'utf-8', errors='replace')
             del globals.GApp.vboximages[name]
             globals.GApp.syncConf()
-            
+
     def slotVBoxImageSelectionChanged(self):
         """ Load VBox settings into the GUI when selecting an entry in the list of VBox images
         """
@@ -413,21 +450,41 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
 
             conf = globals.GApp.vboximages[name]
             
-            self.comboBoxNameVBoxImage.setItemText(0 ,name)
+            self.comboBoxNameVBoxImage.setItemText(0 , conf.filename)
             self.comboBoxNameVBoxImage.setCurrentIndex(0)
-            self.VBoxImage.setText(conf.filename)
-            
+            self.VBoxID.setText(name)
+
             if self.conf.enable_GuestControl:
                 self.VBoxGuestControl_User.setEnabled(True)
                 self.VBoxGuestControl_Password.setEnabled(True)
             else:
                 self.VBoxGuestControl_User.setEnabled(False)
                 self.VBoxGuestControl_Password.setEnabled(False)
-                
+
+            if conf.first_nic_managed:
+                self.checkBoxVBoxFirstInterfaceManaged.setCheckState(QtCore.Qt.Checked)
+            else:
+                self.checkBoxVBoxFirstInterfaceManaged.setCheckState(QtCore.Qt.Unchecked)
+
+            if conf.headless_mode:
+                self.checkBoxVBoxHeadlessMode.setCheckState(QtCore.Qt.Checked)
+            else:
+                self.checkBoxVBoxHeadlessMode.setCheckState(QtCore.Qt.Unchecked)
+
+            if conf.console_support:
+                self.checkBoxVboxConsoleSupport.setCheckState(QtCore.Qt.Checked)
+            else:
+                self.checkBoxVboxConsoleSupport.setCheckState(QtCore.Qt.Unchecked)
+
+            if conf.console_telnet_server:
+                self.checkBoxVboxConsoleServer.setCheckState(QtCore.Qt.Checked)
+            else:
+                self.checkBoxVboxConsoleServer.setCheckState(QtCore.Qt.Unchecked)
+
             self.VBoxGuestControl_User.setText(conf.guestcontrol_user)
             self.VBoxGuestControl_Password.setText(conf.guestcontrol_password)
             self.VBoxNICNb.setValue(conf.nic_nb)
-        
+
             index = self.VBoxNIC.findText(conf.nic)
             if index != -1:
                 self.VBoxNIC.setCurrentIndex(index)
@@ -463,6 +520,7 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
             if globals.GApp.systconf['vbox'].enable_VBoxManager:
                 host = globals.GApp.systconf['vbox'].VBoxManager_binding
                 port = globals.GApp.systconf['vbox'].vboxwrapper_port
+
                 if globals.GApp.VBoxManager.startVBox(port) == False:
                     self.labelVBoxStatus.setText('<font color="red">' + translate("UiConfig_PreferencesVirtualBox", "Failed to start VBoxwrapper")  + '</font>')
                     return
@@ -510,6 +568,6 @@ class UiConfig_PreferencesVirtualBox(QtGui.QWidget, Ui_PreferencesVirtualBox):
                     self.labelVBoxStatus.setText('<font color="brown">' + translate("UiConfig_PreferencesVirtualBox", "Failed to start xdotool")  + '</font>')
                     return
 
-            self.labelVBoxStatus.setText('<font color="green">' + translate("UiConfig_PreferencesVirtualBox", "VBoxwrapper and VirtualBox API %s have successfully started") % vbox_version  + '</font>')
+            self.labelVBoxStatus.setText('<font color="green">' + translate("UiConfig_PreferencesVirtualBox", "VBoxwrapper and VirtualBox API %s<br>have successfully started") % vbox_version  + '</font>')
             self.fillVMnames(vbox)
             globals.GApp.VBoxManager.stopVBox()
\ No newline at end of file
