@@ -25,6 +25,7 @@ import GNS3.Dynagen.dynamips_lib as lib
 import GNS3.Globals as globals
 import GNS3.UndoFramework as undo
 import GNS3.WindowManipulator as winm
+import GNS3.Dynagen.portTracker_lib as tracker
 from PyQt4 import QtGui, QtCore, QtNetwork
 from PyQt4.QtGui import QMainWindow, QIcon
 from GNS3.Ui.Form_MainWindow import Ui_MainWindow
@@ -33,12 +34,12 @@ from GNS3.IOSDialog import IOSDialog
 from GNS3.SymbolManager import SymbolManager
 from GNS3.ProjectDialog import ProjectDialog
 from GNS3.SnapshotDialog import SnapshotDialog
-from GNS3.Utils import debug, translate, fileBrowser
+from GNS3.Utils import debug, translate, fileBrowser, showDetailedMsgBox
 from GNS3.Config.Preferences import PreferencesDialog
 from GNS3.Config.Objects import recentFilesConf
 from GNS3.Node.IOSRouter import IOSRouter
 from GNS3.Node.AnyEmuDevice import AnyEmuDevice, JunOS, IDS, QemuDevice
-from GNS3.Node.AnyVBoxEmuDevice import AnyVBoxEmuDevice, VBoxDevice
+from GNS3.Node.AnyVBoxEmuDevice import AnyVBoxEmuDevice
 from GNS3.Pixmap import Pixmap
 
 class Workspace(QMainWindow, Ui_MainWindow):
@@ -118,6 +119,13 @@ class Workspace(QMainWindow, Ui_MainWindow):
                 self.__action_CheckForUpdate(silent=True)
                 globals.GApp.systconf['general'].last_check_for_update = currentEpoch
 
+        # Port tracker 
+        self.track = tracker.portTracker()
+        # Register local addresses into tracker
+        local_addresses = map(lambda addr: unicode(addr.toString()), QtNetwork.QNetworkInterface.allAddresses())
+        for addr in local_addresses:
+            self.track.addLocalAddress(addr)
+
     def __connectActions(self):
         """ Connect all needed pair (action, SIGNAL)
         """
@@ -186,7 +194,9 @@ class Workspace(QMainWindow, Ui_MainWindow):
         self.menu_View.addMenu(self.submenu_Docks)
 
         # Create and populate recent files submenu
-        for recent_file_conf in globals.GApp.recentfiles:
+        recent_files = list(globals.GApp.recentfiles)
+        recent_files.reverse()
+        for recent_file_conf in recent_files:
             action = QtGui.QAction(recent_file_conf.path, self.submenu_RecentFiles)
             self.submenu_RecentFiles.addAction(action)
 
@@ -373,9 +383,9 @@ class Workspace(QMainWindow, Ui_MainWindow):
         self.projectConfigs = None
 
         globals.GApp.topology.clear()
-
         self.clear_workdir(projectWorkdir)
         globals.GApp.mainWindow.capturesDock.refresh()
+        self.track.clearAllTcpPort()
 
     def __action_Clear(self):
         """ Clear the topology
@@ -422,11 +432,22 @@ class Workspace(QMainWindow, Ui_MainWindow):
         fb = fileBrowser(translate('Workspace', 'Directory to write startup-configs'), directory=os.path.normpath(globals.GApp.systconf['general'].project_path), parent=self)
         path = fb.getDir()
         if path:
+            path = os.path.normpath(path)
             globals.GApp.workspace.projectConfigs = path
             net = netfile.NETFile()
-            for device in globals.GApp.dynagen.devices.values():
-                if isinstance(device, lib.Router):
-                    net.export_router_config(device)
+
+            for node in globals.GApp.topology.items():
+                # record router configs
+                if isinstance(node, IOSRouter) and globals.GApp.workspace.projectConfigs:
+                    device = node.get_dynagen_device()
+                    try:
+                        net.export_router_config(device)
+                    except lib.DynamipsErrorHandled:
+                        node.shutdownInterfaces()
+                        node.state = device.state
+                        node.updateToolTips()
+                        globals.GApp.mainWindow.treeWidget_TopologySummary.changeNodeStatus(node.hostname, node.state)
+                        continue
 
     def importConfigs(self):
         """ Import all startup-config
@@ -435,6 +456,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
         fb = fileBrowser(translate('Workspace', 'Directory to read startup-configs'), directory=os.path.normpath(globals.GApp.systconf['general'].project_path), parent=self)
         path = fb.getDir()
         if path:
+            path = os.path.normpath(path)
             try:
                 contents = os.listdir(path)
             except OSError, e:
@@ -490,6 +512,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
                                         filter = 'PNG File (*.png);;GIF File (*.gif);;JPG File (*.jpeg *.jpg);;BMP File (*.bmp);;XPM File (*.xpm *.xbm);;PBM File (*.pbm);;PGM File (*.pgm);;PPM File (*.ppm);;All files (*.*)',
                                         directory=directory, parent=self).getFile()
         if path != None and path != '':
+            path = os.path.normpath(path)
             pixmap_image = QtGui.QPixmap(path)
             if not pixmap_image.isNull():
 
@@ -510,6 +533,16 @@ class Workspace(QMainWindow, Ui_MainWindow):
                 command = undo.AddItem(globals.GApp.topology, item, translate('Workspace', 'picture'))
                 globals.GApp.topology.undoStack.push(command)
 
+    def stopAction_addLink(self):
+        """ Stop the add link action (called from the Scene)
+        """
+
+        self.action_Add_link.setChecked(False)
+        self.action_Add_link.setText(translate('Workspace', 'Add a link'))
+        self.action_Add_link.setIcon(QIcon(':/icons/connection.svg'))
+        globals.addingLinkFlag = False
+        globals.GApp.scene.setCursor(QtCore.Qt.ArrowCursor)
+
     def __action_addLink(self):
         """ Implement the QAction `addLink'
         - This function manage the creation of a connection between two nodes.
@@ -719,6 +752,8 @@ class Workspace(QMainWindow, Ui_MainWindow):
             text: string
         """
 
+        errors = ""
+        translated_action = ""
         node_list = []
         if autostart == True:
             for (hostname,value) in globals.GApp.dynagen.autostart.iteritems():
@@ -734,43 +769,52 @@ class Workspace(QMainWindow, Ui_MainWindow):
         if count == 0:
             return
         progress = QtGui.QProgressDialog(text, translate("Workspace", "Abort"), 0, count, globals.GApp.mainWindow)
+        progress.setWindowTitle("GNS3")
         progress.setMinimum(1)
         progress.setWindowModality(QtCore.Qt.WindowModal)
         globals.GApp.processEvents(QtCore.QEventLoop.AllEvents)
         current = 0
         for node in node_list:
+            server = node.get_dynagen_device().dynamips.host + ':' + str(node.get_dynagen_device().dynamips.port)
             progress.setValue(current)
             globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
             if progress.wasCanceled():
                 progress.reset()
                 break
             try:
-                if action == 'start':
+                if action == 'starting':
+                    translated_action = translate("Workspace", "starting")
                     node.startNode(progress=True)
                     # Slow start feature
                     seconds = globals.GApp.systconf['general'].slow_start
                     if seconds > 0:
                         globals.GApp.processEvents(QtCore.QEventLoop.AllEvents | QtCore.QEventLoop.WaitForMoreEvents, 1000)
                         time.sleep(seconds)
-                if action == 'stop':
+                if action == 'stopping':
+                    translated_action = translate("Workspace", "stopping")
                     node.stopNode(progress=True)
-                if action == 'suspend':
+                if action == 'suspending':
+                    translated_action = translate("Workspace", "suspending")
                     node.suspendNode(progress=True)
-                if action == 'reload':
+                if action == 'reloading':
+                    translated_action = translate("Workspace", "reloading")
                     node.reloadNode(progress=True)
             except lib.DynamipsError, msg:
-                QtGui.QMessageBox.critical(self, translate("Workspace", "%s: Dynamips error") % node.hostname,  unicode(msg))
-            except lib.DynamipsWarning,  msg:
-                QtGui.QMessageBox.warning(self, translate("Workspace", "%s: Dynamips warning") % node.hostname,  unicode(msg))
-                continue
+                errors += translate("Workspace", "%s: error from server %s: %s") % (node.hostname, server, unicode(msg))
+                errors += "\n"
+            except lib.DynamipsWarning, msg:
+                errors += translate("Workspace", "%s: warning from server %s: %s") % (node.hostname, server, unicode(msg))
+                errors += "\n"
             except (lib.DynamipsErrorHandled,  socket.error):
-                QtGui.QMessageBox.critical(self, translate("Workspace", "%s: Dynamips error") % node.hostname, translate("Workspace", "Connection lost"))
-                progress.reset()
-                return
-            current += 1
+                errors += translate("Workspace", "%s: lost communication with server %s") % (node.hostname, server)
+                errors += "\n"
+            finally:
+                current += 1
         progress.setValue(count)
         progress.deleteLater()
         progress = None
+        if errors:
+            showDetailedMsgBox(self, translate("Workspace", "%s nodes") % translated_action, translate("Workspace", "Issues have been detected while %s nodes, please check details ...") % translated_action, errors)
 
     def __action_ShowVirtualBoxManager(self):
         """ Show VirtualBox Manager
@@ -804,25 +848,25 @@ class Workspace(QMainWindow, Ui_MainWindow):
         """ Start all nodes
         """
 
-        self.__launchProgressDialog('start', translate("Workspace", "Starting nodes ..."))
+        self.__launchProgressDialog('starting', translate("Workspace", "Starting nodes ..."))
 
     def __action_StopAll(self):
         """ Stop all nodes
         """
 
-        self.__launchProgressDialog('stop', translate("Workspace", "Stopping nodes ..."))
+        self.__launchProgressDialog('stopping', translate("Workspace", "Stopping nodes ..."))
 
     def __action_SuspendAll(self):
         """ Suspend all nodes
         """
 
-        self.__launchProgressDialog('suspend', translate("Workspace", "Suspending nodes ..."))
+        self.__launchProgressDialog('suspending', translate("Workspace", "Suspending nodes ..."))
 
     def __action_ReloadAll(self):
         """ Reload all nodes
         """
 
-        self.__launchProgressDialog('reload', translate("Workspace", "Reloading nodes ..."))
+        self.__launchProgressDialog('reloading', translate("Workspace", "Reloading nodes ..."))
 
     def __action_Help(self):
         """ Launch a browser for the pointing to the documentation page
@@ -866,6 +910,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
         text.replace("%VERSION%", VERSION)
         ui.aboutText.setText(text)
 
+        dialog.setModal(True)
         dialog.show()
         self.centerDialog(dialog)
         dialog.exec_()
@@ -894,6 +939,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
         """
 
         from __main__ import VERSION
+        from distutils.version import LooseVersion
 
         network_reply = self.sender()
         isSilent = network_reply.request().attribute(QtNetwork.QNetworkRequest.User).toBool()
@@ -909,15 +955,21 @@ class Workspace(QMainWindow, Ui_MainWindow):
         else:
             latest_release = str(network_reply.readAll()).rstrip()
 
-            if VERSION < latest_release:
-                reply = QtGui.QMessageBox.question(self, translate("Workspace", "Check For Update"),
-                                               translate("Workspace", "Newer GNS3 version %s is available, do you want to visit our website to download it?") % latest_release, QtGui.QMessageBox.Yes, \
-                                               QtGui.QMessageBox.No)
-                if reply == QtGui.QMessageBox.Yes:
-                    QtGui.QDesktopServices.openUrl(QtCore.QUrl("http://www.gns3.net/download"))
-
-            elif not isSilent:
-                QtGui.QMessageBox.information(self, translate("Workspace", "Check For Update"), translate("AbstractNode", "GNS3 is up-to-date!"))
+            try:
+                if LooseVersion(VERSION) < latest_release:
+                    reply = QtGui.QMessageBox.question(self, translate("Workspace", "Check For Update"),
+                                                   translate("Workspace", "Newer GNS3 version %s is available, do you want to visit our website to download it?") % latest_release, QtGui.QMessageBox.Yes, \
+                                                   QtGui.QMessageBox.No)
+                    if reply == QtGui.QMessageBox.Yes:
+                        QtGui.QDesktopServices.openUrl(QtCore.QUrl("http://www.gns3.net/download"))
+    
+                elif not isSilent:
+                    QtGui.QMessageBox.information(self, translate("Workspace", "Check For Update"), translate("AbstractNode", "GNS3 is up-to-date!"))
+            except:
+                # File "GNS3\Workspace.pyo", line 957, in __processCheckForUpdateReply
+                # File "distutils\version.pyo", line 296, in __cmp__
+                #AttributeError: LooseVersion instance has no attribute 'version'
+                debug("Couldn't check for an update, exception in LooseVersion()!")
 
         network_reply.deleteLater()
 
@@ -935,17 +987,19 @@ class Workspace(QMainWindow, Ui_MainWindow):
 
         if file == None:
             return
-
-        path = os.path.abspath(file)
+        path = unicode(os.path.abspath(file))
+        if not os.path.exists(path):
+            QtGui.QMessageBox.critical(self, translate("Workspace", "Loading"), translate("Workspace", "No such file: %s") % file)
+            return
         if not os.path.isfile(path):
-            QtGui.QMessageBox.critical(self, translate("Workspace", "Loading"), translate("Workspace", "Invalid file %s") % file)
+            QtGui.QMessageBox.critical(self, translate("Workspace", "Loading"), translate("Workspace", "Not a regular file: %s") % file)
             return
         self.projectFile = path
         self.setWindowTitle("GNS3 - " + self.projectFile)
         net = netfile.NETFile()
         globals.GApp.scene.resetMatrix()
         net.import_net_file(path)
-        self.__launchProgressDialog('start', translate("Workspace", "Starting nodes ..."), autostart=True)
+        self.__launchProgressDialog('starting', translate("Workspace", "Starting nodes ..."), autostart=True)
 
     def __action_NewProject(self):
         """ Create a new project
@@ -987,6 +1041,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
             new_project = False
         projectDialog = ProjectDialog(self, self.projectFile, self.projectWorkdir, self.projectConfigs, new_project)
         projectDialog.pushButtonOpenProject.setEnabled(False)
+        projectDialog.pushButtonRecentFiles.setEnabled(False)
         if self.projectFile:
             projectDialog.setWindowTitle("Save Project As...")
         #self.projectWorkdir = None
@@ -1024,7 +1079,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
                         try:
                             shutil.copy(node.router.cnfg, self.projectConfigs)
                         except (OSError, IOError), e:
-                            debug("Warning: cannot copy " + file + " to " + self.projectConfigs)
+                            debug("Warning: cannot copy " + node.router.cnfg + " to " + self.projectConfigs)
                             continue
                         except:
                             continue
@@ -1059,7 +1114,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
                         dynamips_files += glob.glob(os.path.normpath(node.hypervisor.workingdir) + os.sep + node.get_platform() + '_' + node.hostname + '_slot*')
                         dynamips_files += glob.glob(os.path.normpath(node.hypervisor.workingdir) + os.sep + node.get_platform() + '_' + node.hostname + '_rom')
                         dynamips_files += glob.glob(os.path.normpath(node.hypervisor.workingdir) + os.sep + node.get_platform() + '_' + node.hostname + '_*flash*')
-                        dynamips_files += [os.path.normpath(node.hypervisor.workingdir) + os.sep + node.get_dynagen_device().formatted_ghost_file()]
+                        #dynamips_files += [os.path.normpath(node.hypervisor.workingdir) + os.sep + node.get_dynagen_device().formatted_ghost_file()]
 
                         for file in dynamips_files:
                             try:
@@ -1071,7 +1126,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
                                 continue
 
                         # clean the original working directory
-                        self.clear_workdir(os.path.normpath(node.hypervisor.workingdir))
+                        #self.clear_workdir(os.path.normpath(node.hypervisor.workingdir))
 
                     if (isinstance(node, QemuDevice) or isinstance(node, JunOS) or isinstance(node, IDS)) and unbase:
                         node.get_dynagen_device().unbase()
@@ -1097,7 +1152,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
                 except lib.DynamipsError, msg:
                     QtGui.QMessageBox.critical(self, translate("Workspace", "Dynamips error %s: %s") % (self.projectWorkdir, unicode(msg)))
 
-        self.__action_Save(auto=True)
+        self.__action_Save()
         self.setWindowTitle("GNS3 Project - " + self.projectFile)
 
     def __action_Snapshot(self):
@@ -1105,6 +1160,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
         """
 
         snapDialog = SnapshotDialog()
+        snapDialog.setModal(True)
         snapDialog.show()
         self.centerDialog(snapDialog)
         snapDialog.exec_()
@@ -1183,7 +1239,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
         save_projectFile = self.projectFile
         self.projectConfigs = snapshot_configs
         self.projectWorkdir = snapshot_workdir
-        self.projectFile = snapshot_dir + os.sep + projectName
+        self.projectFile = unicode(snapshot_dir + os.sep + projectName)
         self.__action_Save(auto=True)
         self.projectFile = save_projectFile
         self.projectConfigs = save_cfg
@@ -1218,6 +1274,59 @@ class Workspace(QMainWindow, Ui_MainWindow):
 
         self.openFile()
 
+    def openFromDroppedFile(self, path):
+        """ Open a .net file from a dropped action
+        """
+
+        if not path.endswith(".net"):
+            QtGui.QMessageBox.critical(self, translate("Workspace", "Message"), translate("Workspace", "The file '%s' has not the right extension (.net)") % os.path.basename(path))
+            return
+
+        if len(globals.GApp.topology.nodes) and globals.GApp.topology.changed == True:
+            reply = QtGui.QMessageBox.question(self, translate("Workspace", "Message"), translate("Workspace", "Would you like to save the current topology?"),
+                                               QtGui.QMessageBox.Yes, QtGui.QMessageBox.No, QtGui.QMessageBox.Cancel)
+            if reply == QtGui.QMessageBox.Yes:
+                self.__action_Save()
+            elif reply == QtGui.QMessageBox.Cancel:
+                return
+
+        self.loadNetfile(path)
+
+    def __addToRecentFiles(self, path):
+        """ Add path to recent files menu
+        """
+
+        # Check is the file is not already in list
+        index = 0
+        for recent_file_conf in globals.GApp.recentfiles:
+            if recent_file_conf.path == path:
+                globals.GApp.recentfiles.pop(index)
+                break
+            index += 1
+
+        # Add to the list
+        if os.path.exists(path):
+            recent_file_conf = recentFilesConf()
+            recent_file_conf.path = path
+            globals.GApp.recentfiles.append(recent_file_conf)
+
+        # Limit number of recent file paths to 10
+        if len(globals.GApp.recentfiles) == 10:
+            globals.GApp.recentfiles.pop(0)
+
+        # Redraw recent files submenu
+        self.submenu_RecentFiles.clear()
+        recent_files = list(globals.GApp.recentfiles)
+        recent_files.reverse()
+        for recent_file_conf in recent_files:
+            action = QtGui.QAction(recent_file_conf.path, self.submenu_RecentFiles)
+            self.submenu_RecentFiles.addAction(action)
+
+        # Need to put back the clear menu action
+        self.submenu_RecentFiles.addSeparator()
+        clear_action = QtGui.QAction(translate("Workspace", "Clear Menu"), self.submenu_RecentFiles)
+        self.submenu_RecentFiles.addAction(clear_action)
+
     def openFile(self):
 
         if globals.GApp.systconf['dynamips'].path == '':
@@ -1229,32 +1338,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
                                        directory=os.path.normpath(globals.GApp.systconf['general'].project_path), parent=self).getFile()
 
         if path and (selected == 'NET file (*.net)' or selected == ''):
-            self.loadNetfile(path)
-
-            # Open recent files code
-
-            # Check is the file is not already in list
-            for recent_file_conf in globals.GApp.recentfiles:
-                if recent_file_conf.path == path:
-                    return
-
-            # Limit number of recent file paths to 10
-            if len(globals.GApp.recentfiles) == 10:
-                globals.GApp.recentfiles.pop(0)
-            recent_file_conf = recentFilesConf()
-            recent_file_conf.path = path
-            globals.GApp.recentfiles.append(recent_file_conf)
-
-            # Redraw recent files submenu
-            self.submenu_RecentFiles.clear()
-            for recent_file_conf in globals.GApp.recentfiles:
-                action = QtGui.QAction(recent_file_conf.path, self.submenu_RecentFiles)
-                self.submenu_RecentFiles.addAction(action)
-
-            # Need to put back the clear menu action
-            self.submenu_RecentFiles.addSeparator()
-            clear_action = QtGui.QAction(translate("Workspace", "Clear Menu"), self.submenu_RecentFiles)
-            self.submenu_RecentFiles.addAction(clear_action)
+            self.loadNetfile(os.path.normpath(path))
 
     def loadNetfile(self, path):
 
@@ -1264,6 +1348,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
             self.projectConfigs = None
             self.projectFile = None
             self.load_netfile(path)
+            self.__addToRecentFiles(path)
             globals.GApp.topology.changed = False
         except IOError, (errno, strerror):
             QtGui.QMessageBox.critical(self, 'Open',  u'Open: ' + strerror)
@@ -1274,7 +1359,8 @@ class Workspace(QMainWindow, Ui_MainWindow):
         """ Autosave feature
         """
 
-        print translate("Workspace", "Auto-saving ... Next one in %s seconds" % str(globals.GApp.systconf['general'].autosave))
+        curtime = time.strftime("%H:%M:%S")
+        print translate("Workspace", "%s: Auto-saving ... Next one in %s seconds" % (curtime, str(globals.GApp.systconf['general'].autosave)))
         self.__action_Save(auto=True)
 
     def __action_Save(self, auto=False):
@@ -1287,6 +1373,7 @@ class Workspace(QMainWindow, Ui_MainWindow):
         try:
             net = netfile.NETFile()
             net.export_net_file(self.projectFile, auto)
+            self.__addToRecentFiles(self.projectFile)
             globals.GApp.topology.changed = False
             autosave = globals.GApp.systconf['general'].autosave
             if autosave > 0:
@@ -1305,12 +1392,14 @@ class Workspace(QMainWindow, Ui_MainWindow):
 
         if path != None and path != '':
             if str(selected) == 'NET file (*.net)' or selected == '':
+                path = os.path.normpath(path)
                 if not path.endswith('.net'):
                     path = path + '.net'
                 self.projectFile = path
                 self.setWindowTitle("GNS3 - " + self.projectFile)
                 net = netfile.NETFile()
                 net.export_net_file(path)
+                self.__addToRecentFiles(path)
                 globals.GApp.topology.changed = False
                 return True
         return False
