@@ -81,9 +81,12 @@ msg = "WELCOME to qemuwrapper.py"
 debugmsg(2, msg)
 
 __author__ = 'Thomas Pani and Jeremy Grossmann'
-__version__ = '0.8.2'
+__version__ = '0.8.3.1'
 
-QEMU_PATH = "qemu"
+if platform.system() == 'Windows':
+    QEMU_PATH = "qemu" # we still use Qemu 0.11.0 on Windows
+else:
+    QEMU_PATH = "qemu-system-i386"
 QEMU_IMG_PATH = "qemu-img"
 PORT = 10525
 IP = ""
@@ -105,7 +108,7 @@ else:
 
 class UDPConnection:
     def __init__(self, saddr, sport, daddr, dport):
-        self.shost = saddr
+        self.saddr = saddr
         self.sport = sport
         self.daddr = daddr
         self.dport = dport
@@ -156,12 +159,18 @@ class xEMUInstance(object):
         debugmsg(2, "xEMUInstance::start()")
         command = self._build_command()
 
-        print "command:", " ".join(command)
+        qemu_cmd = " ".join(command)
+        print "Starting Qemu =>", qemu_cmd
         try:
-            self.process = subprocess.Popen(command,
+            if platform.system() == 'Windows':
+                shell = False
+            else:
+                shell = True
+
+            self.process = subprocess.Popen(qemu_cmd.strip(),
                                             stdin=subprocess.PIPE,
-                                            cwd=self.workdir)
-        except e:
+                                            cwd=self.workdir, shell=shell)
+        except OSError, e:
             print >> sys.stderr, "Unable to start instance", self.name, "of", self.__class__
             print >> sys.stderr, e
             return False
@@ -169,7 +178,13 @@ class xEMUInstance(object):
         # give us some time to wait for Qemu to start
         time.sleep(1)
 
-        print "    pid:", self.process.pid
+        # check if Qemu has exited (not to say crashed!)
+        self.process.poll()
+        if self.process.returncode != None:
+            print >> sys.stderr, "Qemu has exited with return code %i" % self.process.returncode
+            return False
+
+        print "Qemu has successfully started with pid %i" % self.process.pid
 
         if platform.system() == 'Windows':
             print "Setting priority class to BELOW_NORMAL"
@@ -228,20 +243,20 @@ class xEMUInstance(object):
             # fallback on another syntax if the current one is not supported
             if qemuprotocol == 0:
                 try:
-                    p = subprocess.Popen([self.bin, '--help'], stdout = subprocess.PIPE)
+                    p = subprocess.Popen([self.bin, '-help'], stdout = subprocess.PIPE)
                     qemustdout = p.communicate()
                 except:
-                    print >> sys.stderr, "Unable to execute %s --help" % self.bin
+                    print >> sys.stderr, "Unable to execute %s -help" % self.bin
                     return options
                 if not qemustdout[0].__contains__('for dynamips/pemu/GNS3'):
                     print "Falling back to the new qemu syntax"
                     qemuprotocol = 1
             elif qemuprotocol == 1:
                 try:
-                    p = subprocess.Popen([self.bin, '--net', 'socket'], stderr = subprocess.PIPE)
+                    p = subprocess.Popen([self.bin, '-net', 'socket'], stderr = subprocess.PIPE)
                     qemustderr = p.communicate()
                 except:
-                    print >> sys.stderr, "Unable to execute %s --net socket" % self.bin
+                    print >> sys.stderr, "Unable to execute %s -net socket" % self.bin
                     return options
                 if not qemustderr[1].__contains__('udp='):
                     print "Falling back to the old qemu syntax"
@@ -255,10 +270,10 @@ class xEMUInstance(object):
                     options.extend(['-device', '%s,mac=00:00:ab:%02x:%02x:%02d' % (self.netcard, random.randint(0x00, 0xff), random.randint(0x00, 0xff), vlan)])
                 if vlan in self.udp:
                     options.append('-netdev')
-                    options.append('socket,id=gns3-%s,udp=%s:%s,localaddr=%s:%s' % (vlan, self.udp[vlan].shost, self.udp[vlan].sport, self.udp[vlan].daddr, self.udp[vlan].dport))
+                    options.append('socket,id=gns3-%s,udp=%s:%s,localaddr=%s:%s' % (vlan, self.udp[vlan].daddr, self.udp[vlan].dport, self.udp[vlan].saddr, self.udp[vlan].sport))
                 # FIXME: dump relies on vlans, incompatible with the new syntax: patch it.
                 #if vlan in self.capture:
-                    #options.extend(['-net', 'dump,vlan=%s,file=%s' % (vlan, self.capture[vlan])])
+                    #options.extend(['-net', '"dump,vlan=%s,file=%s"' % (vlan, self.capture[vlan])])
             else:
                 options.append('-net')
                 if vlan in self.nic:
@@ -272,7 +287,7 @@ class xEMUInstance(object):
                              self.udp[vlan].dport,
                              self.udp[vlan].daddr)])
                 if vlan in self.capture:
-                    options.extend(['-net', 'dump,vlan=%s,file=%s' % (vlan, self.capture[vlan])])
+                    options.extend(['-net', '"dump,vlan=%s,file=%s"' % (vlan, self.capture[vlan])])
 
         return options
 
@@ -344,12 +359,12 @@ class QEMUInstance(xEMUInstance):
         command.extend(self._disk_options())
         command.extend(self._image_options())
         command.extend(self._kernel_options())
-        if bool(self.kvm) == True:
+        if self.kvm == 'True':
             command.extend(['-enable-kvm'])
         command.extend(self._net_options())
         command.extend(self._ser_options())
         if self.options:
-            command.extend(self.options.split())
+            command.extend([self.options])
         return command
 
     def _disk_options(self):
@@ -398,15 +413,15 @@ class ASAInstance(QEMUInstance):
             except OSError, e:
                 print >> sys.stderr, "Execution failed:", e
 
-        return ('-hda', flash)
+        return ('-hda', '"' + flash + '"')
 
     def _image_options(self):
         debugmsg(3, "ASAInstance::_image_options()")
-        return ('-kernel', self.kernel, '-initrd', self.initrd)
+        return ('-kernel', '"' + self.kernel + '"', '-initrd', '"' + self.initrd + '"')
 
     def _kernel_options(self):
         debugmsg(3, "ASAInstance::_kernel_options()")
-        return  ('-append', self.kernel_cmdline)
+        return  ('-append', '"' + self.kernel_cmdline + '"')
 
 class JunOSInstance(QEMUInstance):
 
@@ -467,7 +482,7 @@ class JunOSInstance(QEMUInstance):
             except OSError, e:
                 print >> sys.stderr, "Execution failed:", e
 
-        return (flash, '-hdb', swap)
+        return ('"' + flash + '"', '-hdb', '"' + swap + '"')
 
 class IDSInstance(QEMUInstance):
 
@@ -541,7 +556,7 @@ class IDSInstance(QEMUInstance):
             except OSError, e:
                 print >> sys.stderr, "Execution failed:", e
 
-        return ('-hda', img1, '-hdb', img2)
+        return ('-hda', '"' + img1 + '"', '-hdb', '"' + img2 + '"')
 
 class QemuDeviceInstance(QEMUInstance):
 
@@ -604,8 +619,7 @@ class QemuDeviceInstance(QEMUInstance):
         debugmsg(3, "flash = %s" % str(flash))
         debugmsg(3, "image = %s" % str(self.image))
         debugmsg(3, "swap = %s" % str(swap))
-
-        return (flash, '-hdb', swap)
+        return ('"' + flash + '"', '-hdb', '"' + swap + '"')
 
 class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
 
@@ -682,15 +696,17 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
             self.handle_one_request()
             while not self.close_connection:
                 self.handle_one_request()
-        except socket.error:
-            pass
+        except socket.error, e:
+            print >> sys.stderr, e
+            self.request.close()
+            return
 
     def __get_tokens(self, request):
         debugmsg(3, "QemuWrapperRequestHandler::__get_tokens(%s)" % str(request))
         input_ = cStringIO.StringIO(request)
         tokens = []
         try:
-            tokens = csv.reader(input_, delimiter=' ').next()
+            tokens = csv.reader(input_, delimiter=' ', escapechar='\\').next()
         except StopIteration:
             pass
         return tokens
@@ -700,6 +716,11 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
         request = self.rfile.readline()
         request = request.rstrip()      # Strip package delimiter.
 
+        # Don't process empty strings (this creates Broken Pipe exceptions)
+        #FIXME: this causes 100% cpu usage on Windows.
+        #if request == "":
+        #    return
+
         # Parse request.
         tokens = self.__get_tokens(request)
         if len(tokens) < 2:
@@ -783,6 +804,7 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
         debugmsg(2, "QemuWrapperRequestHandler::do_qemuwrapper_qemu_path(%s)" % str(data))
         qemu_path, = data
         try:
+            qemu_path = os.path.normpath(qemu_path)
             os.access(qemu_path, os.F_OK)
             global QEMU_PATH
             QEMU_PATH = qemu_path
@@ -798,6 +820,7 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
         debugmsg(2, "QemuWrapperRequestHandler::do_qemuwrapper_qemu_img_path(%s)" % str(data))
         qemu_img_path, = data
         try:
+            qemu_img_path = os.path.normpath(qemu_img_path)
             os.access(qemu_img_path, os.F_OK)
             global QEMU_IMG_PATH
             QEMU_IMG_PATH = qemu_img_path
@@ -813,9 +836,13 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
         debugmsg(2, "QemuWrapperRequestHandler::do_qemuwrapper_working_dir(%s)" % str(data))
         working_dir, = data
         try:
+            working_dir = os.path.normpath(working_dir)
             os.chdir(working_dir)
+            global WORKDIR
+            WORKDIR = working_dir
+            print "Working directory is now %s" % WORKDIR
             for qemu_name in QEMU_INSTANCES.keys():
-                QEMU_INSTANCES[qemu_name].workdir = os.path.join(os.getcwdu(), QEMU_INSTANCES[qemu_name].name)
+                QEMU_INSTANCES[qemu_name].workdir = os.path.join(working_dir, QEMU_INSTANCES[qemu_name].name)
             self.send_reply(self.HSC_INFO_OK, 1, "OK")
         except OSError, e:
             self.send_reply(self.HSC_ERR_INV_PARAM, 1,
@@ -904,6 +931,8 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
             self.send_reply(self.HSC_ERR_UNK_OBJ, 1,
                             "Cannot set attribute '%s' for '%s" % (attr, name))
             return
+        if attr in ("image", "initrd", "kernel", "image1", "image2"):
+            value = os.path.normpath(value)
         print >> sys.stderr, '!! %s.%s = %s' % (name, attr, value)
         setattr(QEMU_INSTANCES[name], attr, value)
         self.send_reply(self.HSC_INFO_OK, 1, "%s set for '%s'" % (attr, name))
@@ -949,7 +978,7 @@ class QemuWrapperRequestHandler(SocketServer.StreamRequestHandler):
                             "unable to find Qemu '%s'" % name)
             return
 
-        QEMU_INSTANCES[name].capture[int(vlan)] = path
+        QEMU_INSTANCES[name].capture[int(vlan)] = os.path.normpath(path)
         self.send_reply(self.HSC_INFO_OK, 1, "OK")
 
     def do_qemu_delete_capture(self, data):
