@@ -96,7 +96,7 @@ class VBoxController_4_1():
         self.guestIP = ""
         self.VBoxBug9239Workaround = True # VBoxSVC crash on Windows hosts.
 
-    def start(self, vmname, nics, udp, capture, netcard):
+    def start(self, vmname, nics, udp, capture, netcard, first_nic_managed='False', headless_mode='False', pipe_name=None):
         debugmsg(2, "VBoxController_4_1::start()")
         # note: If you want to improve 'vboxwrapper' code, take a look at
         #   'vboxshell', the official VirtualBox frontend writen in python.
@@ -105,6 +105,9 @@ class VBoxController_4_1():
         self.udp = udp
         self.capture = capture
         self.netcard = netcard
+        self.first_nic_managed = first_nic_managed
+        self.headless_mode = headless_mode
+        self.pipe_name = pipe_name
         debugmsg(3, "vmname = %s, nics = %s, capture = %s, netcard = %s" % (vmname, nics, capture, netcard))
 
         debugmsg(2, "findMachine() is starting vmname = %s" % unicode(self.vmname))
@@ -119,6 +122,8 @@ class VBoxController_4_1():
             return False
         if not self._safeNetOptions():
             return False
+        if not self._safeConsoleOptions():
+            return False
         if not self._safeLaunchVMProcess():
             return False
         debugmsg(3, "progress.percent = %s" % str(self.progress.percent))
@@ -200,6 +205,21 @@ class VBoxController_4_1():
           return False
         return True
 
+    def setName(self, name):
+        debugmsg(2, "VBoxController_4_1::setName()")
+        try:
+            self.mach.setGuestPropertyValue("NameInGNS3",name)
+        except E_ACCESSDENIED:
+            debugmsg(2, "setName FAILED : E_ACCESSDENIED")
+            return False
+        except VBOX_E_INVALID_VM_STATE:
+            debugmsg(2, "setName FAILED : VBOX_E_INVALID_VM_STATE")
+            return False
+        except VBOX_E_INVALID_OBJECT_STATE:
+            debugmsg(2, "setName FAILED : VBOX_E_INVALID_OBJECT_STATE")
+            return False
+        return True
+
     def displayWindowFocus(self):
         debugmsg(2, "VBoxController_4_1::displayWindowFocus()")
         # For example, look at "VBoxGlobal.cpp"
@@ -276,6 +296,67 @@ class VBoxController_4_1():
                 continue
         return True
 
+    def _console_options(self):
+        """
+        # Example to manually set serial parameters with Python
+
+        from vboxapi import VirtualBoxManager
+        mgr = VirtualBoxManager(None, None)
+        mach = mgr.vbox.findMachine("My VM")
+        session = mgr.mgr.getSessionObject(mgr.vbox)
+        mach.lockMachine(session, 1)
+        mach2=session.machine
+        serial_port = mach2.getSerialPort(0)
+        serial_port.enabled = True
+        serial_port.path = "/tmp/test_pipe"
+        serial_port.hostMode = 1
+        serial_port.server = True
+        session.unlockMachine()
+        
+        """
+        debugmsg(2, "VBoxController_4_1::_console_options()")
+        #This code looks really ulgy due to constant 'try' and 'except' pairs.
+        #But this is because VirtualBox COM interfaces constantly fails
+        #  on slow or loaded hosts. (on both Windows and Linux hosts)
+        #Without 'try/except' pairs it results in vboxwrapper crashes.
+        #
+        #To reproduce: Try to configure several VMs, and restart them all in
+        #  loop on heavily loaded hosts.
+
+        if not self.pipe_name:
+            return True
+
+        if not self._safeLockMachine():
+            return False
+        try:
+            mach2=self.session.machine
+        except:
+            debugmsg(1, "_console_options() -> self.session.machine FAILED !")
+            return False
+
+        try:
+            serial_port = mach2.getSerialPort(0)
+        except:
+            #Usually due to COM Error: "The object is not ready"
+            debugmsg(1, "_console_options() -> getSerialPort() FAILED !")
+            return False
+
+        try:
+            serial_port.enabled = True
+            serial_port.path = self.pipe_name
+            serial_port.hostMode = 1
+            serial_port.server = True
+        except:
+            #Usually due to COM Error: "The object is not ready"
+            debugmsg(1, "_console_options() -> serial port settings FAILED !")
+            return False
+
+        if not self._safeSaveSettings(mach2):
+            return False
+        if not self._safeUnlockMachine():
+            return False
+        return True
+
     def _net_options(self):
         debugmsg(2, "VBoxController_4_1::_net_options()")
         #This code looks really ulgy due to constant 'try' and 'except' pairs.
@@ -285,6 +366,7 @@ class VBoxController_4_1():
         #
         #To reproduce: Try to configure several VMs, and restart them all in
         #  loop on heavily loaded hosts.
+
         if not self._safeLockMachine():
             return False
         try:
@@ -303,13 +385,19 @@ class VBoxController_4_1():
             debugmsg(1, "_net_options() -> getNetworkAdapter() FAILED !")
             return False
 
+        if self.first_nic_managed == 'True':
+            # first nic is managed by GNS3
+            start_nic = 1
+        else:
+            # We leave vNIC #1 (vnic = 0) for VirtualBox management purposes
+            start_nic = 2
 
-        for vnic in range(2, int(self.nics) + 1):
-        #for vnic in range(min(int(self.nics), self.maxNics-1)):
+        for vnic in range(start_nic, int(self.nics) + 1):
             # By design, we leave vNIC #1 (vnic = 0) for VirtualBox management purposes
             try:
                 # Vbox API starts counting from 0
                 netadp = mach2.getNetworkAdapter(vnic-1)
+                #netadp = mach2.getNetworkAdapter(0)
             except:
                 #Usually due to COM Error on loaded hosts: "The object is not ready"
                 debugmsg(1, "_net_options() -> getNetworkAdapter() FAILED !")
@@ -603,7 +691,12 @@ class VBoxController_4_1():
                 debugmsg(1, "launchVMProcess() FAILED, no retries left, giving up...")
                 return False
             try:
-                self.progress = self.mach.launchVMProcess(self.session, "gui", "")
+                if self.headless_mode == 'True':
+                    mode = "headless"
+                else:
+                    mode = "gui"
+                print "Starting %s in %s mode" % (self.vmname, mode)
+                self.progress = self.mach.launchVMProcess(self.session, mode, "")
                 break
             except Exception, e:
                 #This will usually happen if you try to start the same VM twice,
@@ -700,6 +793,24 @@ class VBoxController_4_1():
                 continue
         return True
 
+    def _safeConsoleOptions(self):
+        #_safe*() functions exist as a protection against COM failure on loaded hosts.
+        debugmsg(3, "VBoxController_4_1::_safeConsoleOptions()")
+        #This command is retried several times, because it fails more often...
+        retries=4
+        for retry in range(retries):
+            if retry == (retries-1):
+                debugmsg(1, "_console_options() FAILED, no retries left, giving up...")
+                return False
+            if self._console_options():
+                break
+            else:
+                #fails on heavily loaded hosts...
+                debugmsg(1, "_console_options() FAILED, retrying #%d" % (retry+1))
+                time.sleep(1)
+                continue
+        return True
+
     def _safeLockMachine(self):
         #_safe*() functions exist as a protection against COM failure on loaded hosts.
         debugmsg(3, "VBoxController_4_1::_safeLockMachine()")
