@@ -39,6 +39,12 @@ import sys
 import threading
 import SocketServer
 import time
+import re
+from tcp_pipe_proxy import PipeProxy
+
+if sys.platform.startswith("win"):
+    import win32file
+    import msvcrt
 
 try:
     reload(sys)
@@ -77,7 +83,7 @@ debugmsg(2, 'Starting vboxwrapper')
 debugmsg(1, "debuglevel =  %s" % debuglevel + os.linesep)
 
 __author__ = 'Thomas Pani, Jeremy Grossmann and Alexey Eromenko "Technologov"'
-__version__ = '0.8.2'
+__version__ = '0.8.3.1'
 
 PORT = 11525
 IP = ""
@@ -139,9 +145,15 @@ class xVBOXInstance(object):
         self.netcard = 'automatic'
         self.guestcontrol_user = ''
         self.guestcontrol_password = ''
+        self.first_nic_managed = False
+        self.headless_mode = False
+        self.console_support = False
+        self.console_telnet_server = False
         self.process = None
+        self.pipeThread = None
+        self.pipe = None
         self.workdir = WORKDIR + os.sep + name
-        self.valid_attr_names = ['image',  'console', 'nics', 'netcard', 'guestcontrol_user', 'guestcontrol_password']
+        self.valid_attr_names = ['image',  'console', 'nics', 'netcard', 'guestcontrol_user', 'guestcontrol_password', 'first_nic_managed', 'headless_mode', 'console_support', 'console_telnet_server']
         self.mgr = g_vboxManager
         self.vbox = self.mgr.vbox
         # Future-proof way to control several major versions of VBox:
@@ -165,8 +177,10 @@ class xVBOXInstance(object):
 
     def create(self):
         debugmsg(2, "xVBOXInstance::create()")
-        if not os.path.exists(self.workdir):
-            os.makedirs(self.workdir)
+
+        # VBOX doesn't need a working directory ... for now
+#        if not os.path.exists(self.workdir):
+#            os.makedirs(self.workdir)
 
     def clean(self):
         pass
@@ -178,7 +192,47 @@ class xVBOXInstance(object):
         debugmsg(2, "xVBOXInstance::start()")
         self.vmname = self.image
 
-        return self.vbc.start(self.vmname, self.nics, self.udp, self.capture, self.netcard)
+        if self.console_support == 'True':
+            p = re.compile('\s+', re.UNICODE)
+            pipe_name = p.sub("_", self.vmname)
+            if sys.platform.startswith('win'):
+                pipe_name = r'\\.\pipe\VBOX\%s' % pipe_name
+            elif os.path.exists(self.workdir):
+                pipe_name = self.workdir + os.sep + "vbox_pipe_to_%s" % pipe_name
+            else:
+                pipe_name = "/tmp/vbox_pipe_to_%s" % pipe_name
+        else:
+            pipe_name = None
+
+        started = self.vbc.start(self.vmname, self.nics, self.udp, self.capture, self.netcard, self.first_nic_managed, self.headless_mode, pipe_name)
+
+        if started:
+            self.vbc.setName(self.name)
+
+        if started and self.console_support == 'True' and int(self.console) and self.console_telnet_server == 'True':
+
+            global IP
+            if sys.platform.startswith('win'):
+                try:
+                    self.pipe = open(pipe_name, 'a+b')
+                except:
+                    return started
+                self.pipeThread = PipeProxy(self.vmname, msvcrt.get_osfhandle(self.pipe.fileno()), IP, int(self.console))
+                self.pipeThread.setDaemon(True)
+                self.pipeThread.start()
+            else:
+                try:
+                    self.pipe = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+                    self.pipe.connect(pipe_name)
+                except socket.error, err:
+                    print "connection to pipe %s failed -> %s" % (pipe_name, err[1])
+                    return started
+
+                self.pipeThread = PipeProxy(self.vmname, self.pipe, IP, int(self.console))
+                self.pipeThread.setDaemon(True)
+                self.pipeThread.start()
+
+        return started
 
     def reset(self):
         debugmsg(2, "xVBOXInstance::reset()")
@@ -186,6 +240,19 @@ class xVBOXInstance(object):
 
     def stop(self):
         debugmsg(2, "xVBOXInstance::stop()")
+
+        if self.pipeThread:
+            self.pipeThread.stop()
+            self.pipeThread.join()
+            self.pipeThread = None
+
+        if self.pipe:
+            if sys.platform.startswith('win'):
+                win32file.CloseHandle(msvcrt.get_osfhandle(self.pipe.fileno()))
+            else:
+                self.pipe.close()
+            self.pipe = None
+
         return self.vbc.stop()
 
     def suspend(self):
@@ -415,6 +482,12 @@ class VBoxWrapperRequestHandler(SocketServer.StreamRequestHandler):
     def handle_one_request(self):
         debugmsg(3, "VBoxWrapperRequestHandler::handle_one_request()")
         request = self.rfile.readline()
+
+        # Don't process empty strings (this creates Broken Pipe exceptions)
+        #FIXME: this causes 100% cpu usage on Windows.
+        #if request == "":
+        #    return
+
         debugmsg(3, "handle_one_request(), request = %s" % request)
         # If command exists in cache (=cache hit), we skip further processing
         if self.check_cache(request):
@@ -537,8 +610,11 @@ class VBoxWrapperRequestHandler(SocketServer.StreamRequestHandler):
         working_dir, = data
         try:
             os.chdir(working_dir)
+            global WORKDIR
+            WORKDIR = working_dir
+            print "Working directory is now %s" % WORKDIR
             for vbox_name in VBOX_INSTANCES.keys():
-                VBOX_INSTANCES[vbox_name].workdir = os.path.join(os.getcwdu(), VBOX_INSTANCES[vbox_name].name)
+                VBOX_INSTANCES[vbox_name].workdir = os.path.join(working_dir, VBOX_INSTANCES[vbox_name].name)
             self.send_reply(self.HSC_INFO_OK, 1, "OK")
         except OSError, e:
             self.send_reply(self.HSC_ERR_INV_PARAM, 1,
@@ -564,6 +640,7 @@ class VBoxWrapperRequestHandler(SocketServer.StreamRequestHandler):
         debugmsg(2, "VBoxWrapperRequestHandler::do_vbox_version(%s)" % unicode(data))
 
         global g_vboxManager, VBOXVER, VBOXVER_REQUIRED
+
         if g_vboxManager:
             vboxver_maj = VBOXVER.split('.')[0]
             vboxver_min = VBOXVER.split('.')[1]
@@ -711,13 +788,7 @@ class VBoxWrapperRequestHandler(SocketServer.StreamRequestHandler):
             self.send_reply(self.HSC_ERR_UNK_OBJ, 1,
                             "unable to find VBox '%s'" % name)
             return
-        # ***************
-        #Try to delete UDP:
         VBOX_INSTANCES[name].delete_udp(vnic)
-        #if not VBOX_INSTANCES[name].delete_udp(vnic):
-        #    self.send_reply(self.HSC_ERR_DELETE, 1,
-        #                    "unable to remove UDP connection '%s'" % vnic)
-        #    return
         if VBOX_INSTANCES[name].udp.has_key(int(vnic)):
             del VBOX_INSTANCES[name].udp[int(vnic)]
         self.send_reply(self.HSC_INFO_OK, 1, "OK")
@@ -865,9 +936,17 @@ def main():
     parser.add_option("-p", "--port", type="int", dest="port", help="Port number (default is 11525)")
     parser.add_option("-w", "--workdir", dest="wd", help="Working directory (default is current directory)")
     parser.add_option("-6", "--forceipv6", dest="force_ipv6", help="Force IPv6 usage (default is false; i.e. IPv4)")
-    parser.add_option("-n", "--no-vbox-checks", action="store_true", dest="no_vbox_checks", default=False, help="Do not check for vboxapi loadin and VirtualBox version")
+    parser.add_option("-n", "--no-vbox-checks", action="store_true", dest="no_vbox_checks", default=False, help="Do not check for vboxapi loading and VirtualBox version")
+
+    # ignore an option automatically given by Py2App
+    if sys.platform.startswith('darwin') and len(sys.argv) > 1 and sys.argv[1].startswith("-psn"):
+        del sys.argv[1]
 
     try:
+        # trick to ignore an option automatically given by Py2App
+        #if sys.platform.startswith('darwin') and hasattr(sys, "frozen"):
+        #    (options, args) = parser.parse_args(sys.argv[2:])
+        #else:
         (options, args) = parser.parse_args()
     except SystemExit:
         sys.exit(1)
